from typing import Optional, List
from decimal import Decimal

from validator_collection import validators

from highcharts_core import errors
from highcharts_core.decorators import class_sensitive
from highcharts_core.metaclasses import HighchartsMeta
from highcharts_core.utility_classes.gradients import Gradient
from highcharts_core.utility_classes.patterns import Pattern
from highcharts_core.utility_classes.data_labels import DataLabel, NodeDataLabel


class NodeOptions(HighchartsMeta):
    """Configuration of options for nodes in an Arc Diagram that are associated with a
    specific :class:`ArcDiagramSeries` by the :meth:`ArcDiagramSeries.id`."""

    def __init__(self, **kwargs):
        self._color = None
        self._color_index = None
        self._data_labels = None
        self._id = None
        self._name = None
        self._offset_horizontal = None
        self._offset_vertical = None

        self.color = kwargs.get('color', None)
        self.color_index = kwargs.get('color_index', None)
        self.data_labels = kwargs.get('data_labels', None)
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.offset_horizontal = kwargs.get('offset_horizontal', None)
        self.offset_vertical = kwargs.get('offset_vertical', None)

    @property
    def color(self) -> Optional[str | Gradient | Pattern]:
        """The color of the auto-generated node. Defaults to :obj:`None <python:None>`.

        :rtype: :obj:`None <python:None>`, :class:`Gradient`, :class:`Pattern`, or
          :class:`str <python:str>`
        """
        return self._color

    @color.setter
    def color(self, value):
        from highcharts_core import utility_functions
        self._color = utility_functions.validate_color(value)

    @property
    def color_index(self) -> Optional[int]:
        """When operating in :term:`styled mode`, a specific color index to use for the
        auto-generated node. Defaults to :obj:`None <python:None>`.

        :rtype: :class:`int <python:int>` or :obj:`None <python:None>`
        """
        return self._color_index

    @color_index.setter
    def color_index(self, value):
        self._color_index = validators.integer(value,
                                               allow_empty = True,
                                               minimum = 0)

    @property
    def data_labels(self) -> Optional[DataLabel]:
        """Options for the node's data label.

        :rtype: :class:`DataLabel`, :class:`list <python:list>` of :class:`DataLabel`, or
          :obj:`None <python:None>`
        """
        return self._data_labels

    @data_labels.setter
    @class_sensitive(DataLabel)
    def data_labels(self, value):
        self._data_labels = value

    @property
    def id(self) -> Optional[str]:
        """The id of the auto-generated node, refering to the ``from`` or ``to`` setting
        of the link. Defaults to :obj:`None <python:None>`.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._id

    @id.setter
    def id(self, value):
        self._id = validators.string(value, allow_empty = True)

    @property
    def name(self) -> Optional[str]:
        """The name to display for the node in data labels and tooltips. Defaults to
        :obj:`None <python:None>`.

        .. hint::

          Use this when the name is different from the :meth:`NodeOptions.id`. Where the
          ``id`` must be unique for each node, this is not necessary for the name.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._name

    @name.setter
    def name(self, value):
        self._name = validators.string(value, allow_empty = True)

    @property
    def offset_horizontal(self) -> Optional[str | int | float | Decimal]:
        """The horizontal offset of a node, expressed in either pixels or as a percentage
        of the node size. Defaults to :obj:`None <python:None>`.

        .. hint::

          Positive values shift the node right, negative shift it left.

        :rtype: numeric or :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._offset_horizontal

    @offset_horizontal.setter
    def offset_horizontal(self, value):
        if value is None:
            self._offset_horizontal = None
        else:
            if isinstance(value, str):
                value = value.lower()
                if '%' not in value:
                    raise errors.HighchartsValueError(f'offset_horizontal must be either '
                                                      f'a numeric value, or a percentage '
                                                      f'string. "%" was not found in: '
                                                      f'{value}')
            else:
                value = validators.numeric(value)

            self._offset_horizontal = value

    @property
    def offset_vertical(self) -> Optional[str | int | float | Decimal]:
        """The vertical offset of a node, expressed in either pixels or as a percentage
        of the node size. Defaults to :obj:`None <python:None>`.

        .. hint::

          Positive values shift the node down, negative shift it up.

        :rtype: numeric or :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._offset_vertical

    @offset_vertical.setter
    def offset_vertical(self, value):
        if value is None:
            self._offset_vertical = None
        else:
            if isinstance(value, str):
                value = value.lower()
                if '%' not in value:
                    raise errors.HighchartsValueError(f'offset_vertical must be either '
                                                      f'a numeric value, or a percentage '
                                                      f'string. "%" was not found in: '
                                                      f'{value}')
            else:
                value = validators.numeric(value)

            self._offset_vertical = value

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'color': as_dict.get('color', None),
            'color_index': as_dict.get('colorIndex', None),
            'data_labels': as_dict.get('dataLabels', None),
            'id': as_dict.get('id', None),
            'name': as_dict.get('name', None),
            'offset_horizontal': as_dict.get('offsetHorizontal', None),
            'offset_vertical': as_dict.get('offsetVertical', None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'color': self.color,
            'colorIndex': self.color_index,
            'dataLabels': self.data_labels,
            'id': self.id,
            'name': self.name,
            'offsetHorizontal': self.offset_horizontal,
            'offsetVertical': self.offset_vertical,
        }

        return untrimmed


class DependencyWheelNodeOptions(NodeOptions):
    """Variant of :class:`NodeOptions` for use in :class:`DependencyWheelSeries`."""

    def __init__(self, **kwargs):
        self._column = None
        self._level = None

        self.column = kwargs.get('column', None)
        self.level = kwargs.get('level', None)

        super().__init__(**kwargs)

    @property
    def column(self) -> Optional[int]:
        """An optional column index of where to place the node. Defaults to
        :obj:`None <python:None>`, which places it next to the preceding node.

        .. warning::

          This option name is counter-intuitive in inverted charts, like for example an
          organization chart rendered top-down. In this case the "columns" are rendered
          horizontally, more like "rows".

        :rtype: :class:`int <python:int>` or :obj:`None <python:None>`
        """
        return self._column

    @column.setter
    def column(self, value):
        self._column = validators.integer(value, allow_empty = True)

    @property
    def data_labels(self) -> Optional[NodeDataLabel]:
        """Options for the node's data label.

        :rtype: :class:`NodeDataLabel` or :obj:`None <python:None>`
        """
        return self._data_labels

    @data_labels.setter
    @class_sensitive(NodeDataLabel)
    def data_labels(self, value):
        self._data_labels = value

    @property
    def level(self) -> Optional[int]:
        """An optional level index of where to place the node. Defaults to
        :obj:`None <python:None>`, which places it next to the preceding node.

        .. notes:

          Alias of :meth:`DependencyWheelNodeOptions.column`, but in inverted sankeys and
          org charts, the levels are laid out as rows.

        :rtype: :class:`int <python:int>` or :obj:`None <python:None>`
        """
        return self._level

    @level.setter
    def level(self, value):
        self._level = validators.integer(value, allow_empty = True)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'color': as_dict.get('color', None),
            'color_index': as_dict.get('colorIndex', None),
            'data_labels': as_dict.get('dataLabels', None),
            'id': as_dict.get('id', None),
            'name': as_dict.get('name', None),
            'offset_horizontal': as_dict.get('offsetHorizontal', None),
            'offset_vertical': as_dict.get('offsetVertical', None),

            'column': as_dict.get('column', None),
            'level': as_dict.get('level', None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'column': self.column,
            'level': self.level,
        }

        parent_as_dict = super()._to_untrimmed_dict(in_cls = in_cls) or {}
        for key in parent_as_dict:
            untrimmed[key] = parent_as_dict[key]

        return untrimmed


class OrganizationNodeOptions(DependencyWheelNodeOptions):
    """Variant of :class:`NodeOptions` for use in :class:`OrganizationSeries`."""

    def __init__(self, **kwargs):
        self._image = None
        self._layout = None
        self._title = None

        self.image = kwargs.get('image', None)
        self.layout = kwargs.get('layout', None)
        self.title = kwargs.get('title', None)

        super().__init__(**kwargs)

    @property
    def image(self) -> Optional[str]:
        """The URL of an image for the node card, which will be inserted by the default
        :meth:`DataLabel.node_formatter`. Defaults to :obj:`None <python:None>`.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._image

    @image.setter
    def image(self, value):
        if not value:
            self._image = None
        else:
            try:
                value = validators.url(value)
            except ValueError:
                value = validators.path(value)

            self._image = value

    @property
    def layout(self) -> Optional[str]:
        """The layout for the node's children. Defaults to :obj:`None <python:None>`,
        which behaves as ``'normal'``.

        Accepts:

          * ``'normal'`` - renders children spaced evenly below the node
          * ``'hanging'`` - renders children condensed in a vertical hanging fashion below
            the node, allowing for tighter packing of nodes in the resulting diagram

        .. note::

          Unless explicitly overridden, nodes whose parent use a ``'hanging'`` layout will
          *also* apply a ``'hanging'`` layout.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._layout

    @layout.setter
    def layout(self, value):
        if not value:
            self._layout = None
        else:
            value = validators.string(value)
            value = value.lower()
            if value not in ['normal', 'hanging']:
                raise errors.HighchartsValueError(f'layout expects either "normal" or '
                                                  f'"hanging". Received: {value}')

            self._layout = value

    @property
    def title(self) -> Optional[str]:
        """The job title for the node card. Defaults to :obj:`None <python:None>`.

        .. note::

          Will be inserted by the default :meth:`DataLabel.node_formatter`.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._title

    @title.setter
    def title(self, value):
        self._title = validators.string(value, allow_empty = True)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'color': as_dict.get('color', None),
            'color_index': as_dict.get('colorIndex', None),
            'data_labels': as_dict.get('dataLabels', None),
            'id': as_dict.get('id', None),
            'name': as_dict.get('name', None),
            'offset_horizontal': as_dict.get('offsetHorizontal', None),
            'offset_vertical': as_dict.get('offsetVertical', None),

            'column': as_dict.get('column', None),
            'level': as_dict.get('level', None),

            'image': as_dict.get('image', None),
            'layout': as_dict.get('layout', None),
            'title': as_dict.get('title', None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'image': self.image,
            'layout': self.layout,
            'title': self.title,
        }

        parent_as_dict = super()._to_untrimmed_dict(in_cls = in_cls) or {}
        for key in parent_as_dict:
            untrimmed[key] = parent_as_dict[key]

        return untrimmed



# -*- coding: utf-8 -*-

"""
*******************
tests.conftest
*******************

Utility functions that are used to configure Py.Test context.

"""

import os
import pytest


def pytest_addoption(parser):
    """Define options that the parser looks for in the command-line.

    Pattern to use::

      parser.addoption("--cli-option",
                       action="store",
                       default=None,
                       help="cli-option: help text goes here")

    """
    parser.addoption("--inputs",
                     action="store",
                     default="/home/travis/build/highcharts-for-python/highcharts-core/tests/input_files",
                     help=("inputs: the absolute path to the directory where input"
                           " files can be found"))
    parser.addoption("--downloads",
                     action="store",
                     default="true",
                     help=("downloads: set to 'false' to disable tests of chart export "
                           "via the Highsoft-provided Export Server."))
    parser.addoption("--create-output-directory",
                     action = "store",
                     default = "true",
                     help=("create-output-directory: set to 'false' to error if the output "
                           "directory does not exist, otherwise creates it."))
    parser.addoption("--pyspark",
                     action="store",
                     default="false",
                     help=("pyspark: set to 'false' to disable tests of pyspark-related"
                           " functionality, or 'true' to enable those tests. Defaults to"
                           " 'false'"))


def pytest_runtest_makereport(item, call):
    """Connect current incremental test to its preceding parent."""
    # pylint: disable=W0212
    if "incremental" in item.keywords:
        if call.excinfo is not None:
            parent = item.parent
            parent._previousfailed = item


def pytest_runtest_setup(item):
    """Fail test if preceding incremental test failed."""
    if "incremental" in item.keywords:
        previousfailed = getattr(item.parent, "_previousfailed", None)
        if previousfailed is not None:
            pytest.xfail(
                "previous test failed (%s) for reason: %s" % (previousfailed.name,
                                                              previousfailed))



from typing import Optional, List
from decimal import Decimal

from validator_collection import validators

from highcharts_core import constants, errors, utility_functions
from highcharts_core.options.plot_options.generic import GenericTypeOptions
from highcharts_core.utility_classes.gradients import Gradient
from highcharts_core.utility_classes.patterns import Pattern


class PieOptions(GenericTypeOptions):
    """General options to apply to all Pie series types.

    A pie chart is a circular graphic which is divided into slices to illustrate
    numerical proportion.

    .. tabs::

      .. tab:: Pie Chart

        .. figure:: ../../../_static/pie-example.png
          :alt: Pie Example Chart
          :align: center

      .. tab:: Donut Chart

        .. figure:: ../../../_static/pie-example-donut.png
          :alt: Donut Example Chart
          :align: center

    """

    def __init__(self, **kwargs):
        self._border_color = None
        self._border_radius = None
        self._border_width = None
        self._center = None
        self._color_axis = None
        self._color_index = None
        self._color_key = None
        self._colors = None
        self._depth = None
        self._end_angle = None
        self._fill_color = None
        self._ignore_hidden_point = None
        self._inner_size = None
        self._legend_symbol = None
        self._linecap = None
        self._min_size = None
        self._size = None
        self._sliced_offset = None
        self._start_angle = None
        self._thickness = None

        self.border_color = kwargs.get('border_color', None)
        self.border_radius = kwargs.get('border_radius', None)
        self.border_width = kwargs.get('border_width', None)
        self.center = kwargs.get('center', None)
        self.color_axis = kwargs.get('color_axis', None)
        self.color_index = kwargs.get('color_index', None)
        self.color_key = kwargs.get('color_key', None)
        self.colors = kwargs.get('colors', None)
        self.depth = kwargs.get('depth', None)
        self.end_angle = kwargs.get('end_angle', None)
        self.fill_color = kwargs.get('fill_color', None)
        self.ignore_hidden_point = kwargs.get('ignore_hidden_point', None)
        self.inner_size = kwargs.get('inner_size', None)
        self.legend_symbol = kwargs.get('legend_symbol', None)
        self.linecap = kwargs.get('linecap', None)
        self.min_size = kwargs.get('min_size', None)
        self.size = kwargs.get('size', None)
        self.sliced_offset = kwargs.get('sliced_offset', None)
        self.start_angle = kwargs.get('start_angle', None)
        self.thickness = kwargs.get('thickness', None)

        super().__init__(**kwargs)

    @property
    def border_color(self) -> Optional[str | Gradient | Pattern]:
        """The color of the border surrounding each slice. When :obj:`None <python:None>`,
        the border takes the same color as the slice fill. This can be used together with
        a :meth:`border_width <PieOptions.border_width>` to fill drawing gaps created by
        antialiazing artefacts in borderless pies. Defaults to ``'#ffffff'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._border_color

    @border_color.setter
    def border_color(self, value):
        from highcharts_core import utility_functions
        self._border_color = utility_functions.validate_color(value)

    @property
    def border_radius(self) -> Optional[str | int | float | Decimal]:
        """
        .. versionadded:: Highcharts Core for Python v.1.1.0 / Highcharts Core (JS) v.11.0.0
        
          The corner radius of the border surrounding each slice. Defaults to ``3``.
          
          .. note::
          
            A numerical value signifies the value is expressed in pixels. A percentage string like `50%`
            signifies a size relative to the radius and the inner radius.
            
        :rtype: numeric, :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._border_radius

    @border_radius.setter
    def border_radius(self, value):
        if value is None:
            self._border_radius = None
        else:
            try:
                value = validators.string(value)
                if '%' not in value:
                    raise ValueError
            except (TypeError, ValueError):
                value = validators.numeric(value, minimum = 0)

            self._border_radius = value

    @property
    def border_width(self) -> Optional[int | float | Decimal]:
        """The width of the border surrounding each slice. Defaults to ``1``.

        When setting the border width to ``0``, there may be small gaps between the slices
        due to SVG antialiasing artefacts. To work around this, keep the border width at
        ``0.5`` or ``1``, but set the :meth:`border_color <PieOptions.border_color>` to
        :obj:`None <python:None>` instead.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._border_width

    @border_width.setter
    def border_width(self, value):
        self._border_width = validators.numeric(value,
                                                allow_empty = True,
                                                minimum = 0)

    @property
    def center(self) -> Optional[List[str | int | float | Decimal | constants.EnforcedNullType]]:
        """The center of the pie chart relative to the plot area.

        Can be percentages or pixel values. The default behaviour if
        :obj:`None <python:None>` is to center the pie so that all slices and data labels
        are within the plot area. As a consequence, the pie may actually jump around in a
        chart with dynamic values, as the data labels move. In that case, the center
        should be explicitly set, for example to ``["50%", "50%"]``.

        Defaults to ``['50%', '50%']``.

        :rtype: :obj:`None <python:None>` or :class:`list <python:list>` of numeric or
          :class:`str <python:str>` values
        """
        return self._center

    @center.setter
    def center(self, value):
        if not value:
            self._center = None
        else:
            value = validators.iterable(value)
            if len(value) != 2:
                raise errors.HighchartsValueError(f'center expects a 2-member array. '
                                                  f'Received a {len(value)}-member '
                                                  f'array.')
            processed_values = []
            for item in value:
                try:
                    item = validators.string(item)
                    if '%' not in item:
                        raise errors.HighchartsValueError('center expects an array of '
                                                          'numbers or percentage strings.'
                                                          ' No "%" character found.')
                except TypeError:
                    item = validators.numeric(item)

                processed_values.append(item)

            self._center = processed_values

    @property
    def color_axis(self) -> Optional[str | int | bool]:
        """When using dual or multiple color axes, this setting defines which
        :term:`color axis` the particular series is connected to. It refers to either the
        :meth:`ColorAxis.id` or the index of the axis in the :class:`ColorAxis` array,
        with ``0`` being the first. Set this option to ``False`` to prevent a series from
        connecting to the default color axis.

        Defaults to ``0``.

        :rtype: :obj:`None <python:None>` or :class:`str <python:str>` or
          :class:`int <python:int>` or :class:`bool <python:bool>`
        """
        return self._color_axis

    @color_axis.setter
    def color_axis(self, value):
        if value is None:
            self._color_axis = None
        elif value is False:
            self._color_axis = False
        else:
            try:
                self._color_axis = validators.string(value)
            except TypeError:
                self._color_axis = validators.integer(value,
                                                      minimum = 0)

    @property
    def color_index(self) -> Optional[int]:
        """When operating in :term:`styled mode`, a specific color index to use for the
        series, so that its graphic representations are given the class name
        ``highcharts-color-{n}``.

        Defaults to :obj:`None <python:None>`.

        :rtype: :class:`int <python:int>` or :obj:`None <python:None>`
        """
        return self._color_index

    @color_index.setter
    def color_index(self, value):
        self._color_index = validators.integer(value,
                                               allow_empty = True,
                                               minimum = 0)

    @property
    def color_key(self) -> Optional[str]:
        """Determines what data value should be used to calculate point color if
        :meth:`PieOptions.color_axis` is used.

        .. note::

          Requires to set ``min`` and ``max`` if some custom point property is used or if
          approximation for data grouping is set to ``'sum'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._color_key

    @color_key.setter
    def color_key(self, value):
        self._color_key = validators.string(value, allow_empty = True)

    @property
    def colors(self) -> Optional[List[str | Gradient | Pattern]]:
        """A series-specific or series type-specific color set to apply instead of the
        global colors when :meth:`ArcDiagramOptions.color_by_point` is ``True``.

        :rtype: :class:`list <python:list>` of :class:`str <python:str>`,
          :class:`Gradient`, or :class:`Pattern` OR :obj:`None <python:None>`
        """
        return self._colors

    @colors.setter
    def colors(self, value):
        if not value:
            self._colors = None
        else:
            value = validators.iterable(value)
            self._colors = [utility_functions.validate_color(x) for x in value]

    @property
    def depth(self) -> Optional[int | float | Decimal]:
        """The thickness of a 3D pie. Defaults to ``0``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._depth

    @depth.setter
    def depth(self, value):
        self._depth = validators.numeric(value, allow_empty = True)

    @property
    def end_angle(self) -> Optional[int | float | Decimal]:
        """The end angle of the pie in degrees where 0 is top and 90 is right.
        Defaults to :obj:`None <python:None>`, which is equivalent to
        :meth:`start_angle <PieOptions.start_angle>` plus ``360``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._end_angle

    @end_angle.setter
    def end_angle(self, value):
        self._end_angle = validators.numeric(value,
                                             allow_empty = True,
                                             minimum = -360,
                                             maximum = 360)

    @property
    def fill_color(self) -> Optional[str | Gradient | Pattern]:
        """If the total sum of the pie's values is ``0``, the series is represented as an
        empty circle . The ``fill_color`` setting defines the color of that circle.
        Use :meth:`PieOptions.border_width` to set the border thickness.

        Defaults to :obj:`None <python:None>`.

        :rtype: :obj:`None <python:None>`, :class:`Gradient`, or :class:`Pattern`
        """
        return self._fill_color

    @fill_color.setter
    def fill_color(self, value):
        from highcharts_core import utility_functions
        self._fill_color = utility_functions.validate_color(value)

    @property
    def ignore_hidden_point(self) -> Optional[bool]:
        """If ``True``, the series shall be redrawn as if the hidden points were ``null``.
        If ``False``, hidden points will not be displayed but the slice will still be
        drawn as a gap in the pie.

        Defaults to ``True``.

        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._ignore_hidden_point

    @ignore_hidden_point.setter
    def ignore_hidden_point(self, value):
        if value is None:
            self._ignore_hidden_point = None
        else:
            self._ignore_hidden_point = bool(value)

    @property
    def inner_size(self) -> Optional[str | int]:
        """The size of the inner diameter for the pie, expressed as a percentage or pixel
        value. Defaults to ``0``.

        .. hint::

          A size greater than 0 renders a donut chart.

        .. note::

          Percentages are relative to the pie size. Pixel values are given as integers.

        .. warning::

          If :meth:`PieOptions.thickness` is set, this value will be overridden by the
          ``thickness`` setting.

        :rtype: :class:`str <python:str>`, :class:`int <python:int>`, or
          :obj:`None <python:None>`
        """
        return self._inner_size

    @inner_size.setter
    def inner_size(self, value):
        if value is None:
            self._inner_size = None
        else:
            try:
                value = validators.string(value)
                if '%' not in value:
                    raise errors.HighchartsValueError('inner_size expects a number or '
                                                      'percentage ("%") string. No "%" '
                                                      'character found.')
            except TypeError:
                value = validators.integer(value, minimum = 0)

            self._inner_size = value

    @property
    def linecap(self) -> Optional[str]:
        """The SVG value used for the ``stroke-linecap`` and ``stroke-linejoin`` of a line
        graph. Defaults to ``'round'``, which means that lines are rounded in the ends and
        bends.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._linecap

    @linecap.setter
    def linecap(self, value):
        self._linecap = validators.string(value, allow_empty = True)

    @property
    def min_size(self) -> Optional[str | int | float | Decimal]:
        """The minimum size for a pie in response to auto margins, expressed in pixels or
        percentages. Defaults to ``80``.

        .. note::

          The pie will try to shrink to make room for data labels in side the plot area,
          but only to this size.

        :rtype: :class:`str <python:str>`, :class:`int <python:int>`, or
          :obj:`None <python:None>`
        """
        return self._min_size

    @min_size.setter
    def min_size(self, value):
        if value is None:
            self._min_size = None
        else:
            try:
                value = validators.string(value)
                if '%' not in value:
                    raise errors.HighchartsValueError('min_size expects a number or a '
                                                      'percentage string. No "%" '
                                                      'character found.')
            except TypeError:
                value = validators.numeric(value, minimum = 0)

            self._min_size = value

    @property
    def size(self) -> Optional[str | int]:
        """The diameter of the pie relative to the plot area, expressed as a percentage or
        pixel value given as an integer.

        If :obj:`None <python:None>`, scales the pie to the plot area and gives room for
        data labels within the plot area.

        .. note::

          :meth:`PieOptions.sliced_offset` is also included in the default size
          calculation. As a consequence, the size of the pie may vary when points are
          updated and data labels more around. In that case it is best to set a fixed
          value, for example ``"75%"``.

        :rtype: :class:`str <python:str>`, :class:`int <python:int>`, or
          :obj:`None <python:None>`
        """
        return self._size

    @size.setter
    def size(self, value):
        if value is None:
            self._size = None
        else:
            try:
                value = validators.string(value)
                if '%' not in value:
                    raise errors.HighchartsValueError('size expects a number or a '
                                                      'percentage string. No "%" '
                                                      'character found.')
            except TypeError:
                value = validators.integer(value, minimum = 0)

            self._size = value

    @property
    def sliced_offset(self) -> Optional[int | float | Decimal]:
        """If a point is sliced, moved out from the center, how many pixels should it be
        moved? Defaults to ``10``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._sliced_offset

    @sliced_offset.setter
    def sliced_offset(self, value):
        self._sliced_offset = validators.numeric(value, allow_empty = True)

    @property
    def start_angle(self) -> Optional[int | float | Decimal]:
        """The start angle of the dependency wheel, in degrees where ``0`` is up. Defaults
        to ``0``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._start_angle

    @start_angle.setter
    def start_angle(self, value):
        self._start_angle = validators.numeric(value,
                                               allow_empty = True,
                                               minimum = -360,
                                               maximum = 360)

    @property
    def thickness(self) -> Optional[int]:
        """Thickness describing the ring size for a donut type chart, overriding
        :meth:`PieOptions.inner_size`. Defaults to :obj:`None <python:None>`.

        :rtype: :class:`int <python:int>` or :obj:`None <python:None>`
        """
        return self._thickness

    @thickness.setter
    def thickness(self, value):
        self._thickness = validators.integer(value,
                                             allow_empty = True,
                                             minimum = 0)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'accessibility': as_dict.get('accessibility', None),
            'allow_point_select': as_dict.get('allowPointSelect', None),
            'animation': as_dict.get('animation', None),
            'class_name': as_dict.get('className', None),
            'clip': as_dict.get('clip', None),
            'color': as_dict.get('color', None),
            'cursor': as_dict.get('cursor', None),
            'custom': as_dict.get('custom', None),
            'dash_style': as_dict.get('dashStyle', None),
            'data_labels': as_dict.get('dataLabels', None),
            'description': as_dict.get('description', None),
            'enable_mouse_tracking': as_dict.get('enableMouseTracking', None),
            'events': as_dict.get('events', None),
            'include_in_data_export': as_dict.get('includeInDataExport', None),
            'keys': as_dict.get('keys', None),
            'label': as_dict.get('label', None),
            'legend_symbol': as_dict.get('legendSymbol', None),
            'linked_to': as_dict.get('linkedTo', None),
            'marker': as_dict.get('marker', None),
            'on_point': as_dict.get('onPoint', None),
            'opacity': as_dict.get('opacity', None),
            'point': as_dict.get('point', None),
            'point_description_formatter': as_dict.get('pointDescriptionFormatter', None),
            'selected': as_dict.get('selected', None),
            'show_checkbox': as_dict.get('showCheckbox', None),
            'show_in_legend': as_dict.get('showInLegend', None),
            'skip_keyboard_navigation': as_dict.get('skipKeyboardNavigation', None),
            'sonification': as_dict.get('sonification', None),
            'states': as_dict.get('states', None),
            'sticky_tracking': as_dict.get('stickyTracking', None),
            'threshold': as_dict.get('threshold', None),
            'tooltip': as_dict.get('tooltip', None),
            'turbo_threshold': as_dict.get('turboThreshold', None),
            'visible': as_dict.get('visible', None),

            'border_color': as_dict.get('borderColor', None),
            'border_radius': as_dict.get('borderRadius', None),
            'border_width': as_dict.get('borderWidth', None),
            'center': as_dict.get('center', None),
            'color_axis': as_dict.get('colorAxis', None),
            'color_index': as_dict.get('colorIndex', None),
            'color_key': as_dict.get('colorKey', None),
            'colors': as_dict.get('colors', None),
            'depth': as_dict.get('depth', None),
            'end_angle': as_dict.get('endAngle', None),
            'fill_color': as_dict.get('fillColor', None),
            'ignore_hidden_point': as_dict.get('ignoreHiddenPoint', None),
            'inner_size': as_dict.get('innerSize', None),
            'linecap': as_dict.get('linecap', None),
            'min_size': as_dict.get('minSize', None),
            'size': as_dict.get('size', None),
            'sliced_offset': as_dict.get('slicedOffset', None),
            'start_angle': as_dict.get('startAngle', None),
            'thickness': as_dict.get('thickness', None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'borderColor': self.border_color,
            'borderRadius': self.border_radius,
            'borderWidth': self.border_width,
            'center': self.center,
            'colorAxis': self.color_axis,
            'colorIndex': self.color_index,
            'colorKey': self.color_key,
            'colors': self.colors,
            'depth': self.depth,
            'endAngle': self.end_angle,
            'fillColor': self.fill_color,
            'ignoreHiddenPoint': self.ignore_hidden_point,
            'innerSize': self.inner_size,
            'linecap': self.linecap,
            'minSize': self.min_size,
            'size': self.size,
            'slicedOffset': self.sliced_offset,
            'startAngle': self.start_angle,
            'thickness': self.thickness
        }
        parent_as_dict = super()._to_untrimmed_dict(in_cls = in_cls)

        for key in parent_as_dict:
            untrimmed[key] = parent_as_dict[key]

        return untrimmed


class VariablePieOptions(PieOptions):
    """General options to apply to all Variable Pie series types.

    A variable pie series is a two dimensional series type, where each point renders
    an Y and Z value. Each point is drawn as a pie slice where the size (arc) of the
    slice relates to the Y value and the radius of pie slice relates to the Z value.

    .. figure:: ../../../_static/variablepie-example.png
      :alt: Variable Pie Example Chart
      :align: center

    """

    def __init__(self, **kwargs):
        self._max_point_size = None
        self._min_point_size = None
        self._size_by = None
        self._z_max = None
        self._z_min = None

        self.max_point_size = kwargs.get('max_point_size', None)
        self.min_point_size = kwargs.get('min_point_size', None)
        self.size_by = kwargs.get('size_by', None)
        self.z_max = kwargs.get('z_max', None)
        self.z_min = kwargs.get('z_min', None)

        super().__init__(**kwargs)

    @property
    def max_point_size(self) -> Optional[str | int | float | Decimal]:
        """The maximum size of the points' radius in relation to the chart's plot area. If
        a number is provided, it applies in pixels. Defaults to ``'100%'``.

        :rtype: :class:`str <python:str>`, numeric, or :obj:`None <python:None>`
        """
        return self._max_point_size

    @max_point_size.setter
    def max_point_size(self, value):
        if value is None:
            self._max_point_size = None
        else:
            try:
                value = validators.string(value)
                if '%' not in value:
                    raise errors.HighchartsValueError('max_point_size expects a number '
                                                      'or a % string. No "%" character.')
            except TypeError:
                value = validators.numeric(value, allow_empty = True)

            self._max_point_size = value

    @property
    def min_point_size(self) -> Optional[str | int | float | Decimal]:
        """The minimum size of the points' radius in relation to the chart's plot area. If
        a number is provided, it applies in pixels. Defaults to ``'10%'``.

        :rtype: :class:`str <python:str>`, numeric, or :obj:`None <python:None>`
        """
        return self._min_point_size

    @min_point_size.setter
    def min_point_size(self, value):
        if value is None:
            self._min_point_size = None
        else:
            try:
                value = validators.string(value)
                if '%' not in value:
                    raise errors.HighchartsValueError('min_point_size expects a number '
                                                      'or a % string. No "%" character.')
            except TypeError:
                value = validators.numeric(value, allow_empty = True)

            self._min_point_size = value

    @property
    def size_by(self) -> Optional[str]:
        """Whether the pie slice's value should be represented by the ``'area'`` or the
        ``'radius'`` of the slice. Defaults to ``'area'``.

        .. hint::

          The default (``'area'``) corresponds best to the human perception of the size of
          each pie slice.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._size_by

    @size_by.setter
    def size_by(self, value):
        if not value:
            self._size_by = None
        else:
            value = validators.string(value)
            value = value.lower()
            if value not in ['area', 'radius']:
                raise errors.HighchartsValueError(f'size_by expects either "area", or '
                                                  f'"radius". Received: {value}')

            self._size_by = value

    @property
    def z_max(self) -> Optional[int | float | Decimal]:
        """The maximum possible z value for the point's radius calculation. Defaults to
        :obj:`None <python:None>`.

        .. note::

          If the point's Z value is bigger than ``z_max``, the slice will be drawn
          according to the ``z_max`` value.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._z_max

    @z_max.setter
    def z_max(self, value):
        self._z_max = validators.numeric(value, allow_empty = True)

    @property
    def z_min(self) -> Optional[int | float | Decimal]:
        """The minimum possible z value for the point's radius calculation. Defaults to
        :obj:`None <python:None>`.

        .. note::

          If the point's Z value is smaller than ``z_min``, the slice will be drawn
          according to the ``z_min`` value.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._z_min

    @z_min.setter
    def z_min(self, value):
        self._z_min = validators.numeric(value, allow_empty = True)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'accessibility': as_dict.get('accessibility', None),
            'allow_point_select': as_dict.get('allowPointSelect', None),
            'animation': as_dict.get('animation', None),
            'class_name': as_dict.get('className', None),
            'clip': as_dict.get('clip', None),
            'color': as_dict.get('color', None),
            'cursor': as_dict.get('cursor', None),
            'custom': as_dict.get('custom', None),
            'dash_style': as_dict.get('dashStyle', None),
            'data_labels': as_dict.get('dataLabels', None),
            'description': as_dict.get('description', None),
            'enable_mouse_tracking': as_dict.get('enableMouseTracking', None),
            'events': as_dict.get('events', None),
            'include_in_data_export': as_dict.get('includeInDataExport', None),
            'keys': as_dict.get('keys', None),
            'label': as_dict.get('label', None),
            'legend_symbol': as_dict.get('legendSymbol', None),
            'linked_to': as_dict.get('linkedTo', None),
            'marker': as_dict.get('marker', None),
            'on_point': as_dict.get('onPoint', None),
            'opacity': as_dict.get('opacity', None),
            'point': as_dict.get('point', None),
            'point_description_formatter': as_dict.get('pointDescriptionFormatter', None),
            'selected': as_dict.get('selected', None),
            'show_checkbox': as_dict.get('showCheckbox', None),
            'show_in_legend': as_dict.get('showInLegend', None),
            'skip_keyboard_navigation': as_dict.get('skipKeyboardNavigation', None),
            'sonification': as_dict.get('sonification', None),
            'states': as_dict.get('states', None),
            'sticky_tracking': as_dict.get('stickyTracking', None),
            'threshold': as_dict.get('threshold', None),
            'tooltip': as_dict.get('tooltip', None),
            'turbo_threshold': as_dict.get('turboThreshold', None),
            'visible': as_dict.get('visible', None),

            'border_color': as_dict.get('borderColor', None),
            'border_radius': as_dict.get('borderRadius', None),
            'border_width': as_dict.get('borderWidth', None),
            'center': as_dict.get('center', None),
            'color_axis': as_dict.get('colorAxis', None),
            'color_index': as_dict.get('colorIndex', None),
            'color_key': as_dict.get('colorKey', None),
            'colors': as_dict.get('colors', None),
            'depth': as_dict.get('depth', None),
            'end_angle': as_dict.get('endAngle', None),
            'fill_color': as_dict.get('fillColor', None),
            'ignore_hidden_point': as_dict.get('ignoreHiddenPoint', None),
            'inner_size': as_dict.get('innerSize', None),
            'linecap': as_dict.get('linecap', None),
            'min_size': as_dict.get('minSize', None),
            'size': as_dict.get('size', None),
            'sliced_offset': as_dict.get('slicedOffset', None),
            'start_angle': as_dict.get('startAngle', None),
            'thickness': as_dict.get('thickness', None),

            'max_point_size': as_dict.get('maxPointSize', None),
            'min_point_size': as_dict.get('minPointSize', None),
            'size_by': as_dict.get('sizeBy', None),
            'z_max': as_dict.get('zMax', None),
            'z_min': as_dict.get('zMin', None)
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'maxPointSize': self.max_point_size,
            'minPointSize': self.min_point_size,
            'sizeBy': self.size_by,
            'zMax': self.z_max,
            'zMin': self.z_min
        }
        parent_as_dict = super()._to_untrimmed_dict(in_cls = in_cls)

        for key in parent_as_dict:
            untrimmed[key] = parent_as_dict[key]

        return untrimmed



from typing import Optional, List
from decimal import Decimal

from validator_collection import validators

from highcharts_core import constants, errors, utility_functions
from highcharts_core.decorators import class_sensitive
from highcharts_core.metaclasses import HighchartsMeta
from highcharts_core.utility_classes.gradients import Gradient
from highcharts_core.utility_classes.patterns import Pattern


class PaneBackground(HighchartsMeta):
    """Configuration of the background items to display within a :class:`Pane`."""

    def __init__(self, **kwargs):
        self._background_color = None
        self._border_color = None
        self._border_width = None
        self._class_name = None
        self._inner_radius = None
        self._outer_radius = None
        self._shape = None

        self.background_color = kwargs.get('background_color', None)
        self.border_color = kwargs.get('border_color', None)
        self.border_width = kwargs.get('border_width', None)
        self.class_name = kwargs.get('class_name', None)
        self.inner_radius = kwargs.get('inner_radius', None)
        self.outer_radius = kwargs.get('outer_radius', None)
        self.shape = kwargs.get('shape', None)

    @property
    def background_color(self) -> Optional[str | Gradient | Pattern]:
        """The background color or gradient for the pane. Defaults to
        ``'{ "linearGradient": { "x1": 0, "y1": 0, "x2": 0, "y2": 1 }, stops: [[0, "#ffffff"], [1, "#e6e6e6"]] }'``.

        :returns: The backgorund color for the outer chart area.
        :rtype: :class:`str <python:str>`, :class:`Gradient`, :class:`Pattern``, or
          :obj:`None <python:None>`
        """
        return self._background_color

    @background_color.setter
    def background_color(self, value):
        self._background_color = utility_functions.validate_color(value)

    @property
    def border_color(self) -> Optional[str | Gradient | Pattern]:
        """The color of the pane border. Defaults to
        ``'#cccccc'``.

        :returns: The color of the pane's border.
        :rtype: :class:`str <python:str>`, :class:`Gradient`, :class:`Pattern``, or
          :obj:`None <python:None>`

        """
        return self._border_color

    @border_color.setter
    def border_color(self, value):
        self._border_color = utility_functions.validate_color(value)

    @property
    def border_width(self) -> Optional[int | float | Decimal]:
        """The border width (in pixels) applied to the outer chart border. Defaults to
        ``1``.

        :returns: The border width to apply to the pane.
        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._border_width

    @border_width.setter
    def border_width(self, value):
        self._border_width = validators.numeric(value, allow_empty = True)

    @property
    def class_name(self) -> Optional[str]:
        """A classname to apply styling using CSS. Defaults to
        ``'highcharts-pane'``.

        :returns: The classname to apply to enable styling via CSS.
        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._class_name

    @class_name.setter
    def class_name(self, value):
        self._class_name = validators.string(value, allow_empty = True)

    @property
    def inner_radius(self) -> Optional[int | float | Decimal | str]:
        """The inner radius (in pixels if numeric or as a percentage string) applied to
        the pane. Defaults to ``0``.

        :returns: The inner radius of the pane background.
        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._inner_radius

    @inner_radius.setter
    def inner_radius(self, value):
        if value is None or value == '':
            self._inner_radius = None
        else:
            try:
                self._inner_radius = validators.string(value, allow_empty = True)
            except ValueError:
                self._inner_radius = validators.numeric(value, allow_empty = True)

    @property
    def outer_radius(self) -> Optional[int | float | Decimal | str]:
        """The outer radius (in pixels if numeric or as a percentage string) applied to
        a circular pane background. Defaults to
        ``'105%'``.

        :returns: The outer radius of the pane background.
        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._outer_radius

    @outer_radius.setter
    def outer_radius(self, value):
        if value is None or value == '':
            self._outer_radius = None
        else:
            try:
                self._outer_radius = validators.string(value, allow_empty = True)
            except ValueError:
                self._outer_radius = validators.numeric(value, allow_empty = True)

    @property
    def shape(self) -> Optional[str]:
        """The shape of the pane background. Defaults to
        ``'circle'``.

        Accepts:

          * ``'circle'``
          * ``'solid'``
          * ``'arc'``

        .. note::

          When ``'solid'``, the background is circular.
          When ``'arc'``, the background extends only from the min to the max of the value
          axis.

        :rtype: :class:`str` or :obj:`None <python:None>`
        """
        return self._shape

    @shape.setter
    def shape(self, value):
        if not value:
            self._shape = None
        else:
            value = validators.string(value)
            value = value.lower()
            if value not in ['circle', 'solid', 'arc']:
                raise errors.HighchartsValueError(f'shape expects "circle", "solid", or '
                                                  f'"arc". Received: {value}')
            self._shape = value

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'background_color': as_dict.get('backgroundColor', None),
            'border_color': as_dict.get('borderColor', None),
            'border_width': as_dict.get('borderWidth', None),
            'class_name': as_dict.get('className', None),
            'inner_radius': as_dict.get('innerRadius', None),
            'outer_radius': as_dict.get('outerRadius', None),
            'shape': as_dict.get('shape', None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'backgroundColor': self.background_color,
            'borderColor': self.border_color,
            'borderWidth': self.border_width,
            'className': self.class_name,
            'innerRadius': self.inner_radius,
            'outerRadius': self.outer_radius,
            'shape': self.shape
        }

        return untrimmed


class Pane(HighchartsMeta):
    """The pane serves as a container for axes and backgrounds for circular gauges and
    polar charts."""

    def __init__(self, **kwargs):
        self._background = None
        self._center = None
        self._end_angle = None
        self._inner_size = None
        self._size = None
        self._start_angle = None

        self.background = kwargs.get('background', None)
        self.center = kwargs.get('center', None)
        self.end_angle = kwargs.get('end_angle', None)
        self.inner_size = kwargs.get('inner_size', None)
        self.size = kwargs.get('size', None)
        self.start_angle = kwargs.get('start_angle', None)

    @property
    def _dot_path(self) -> Optional[str]:
        """The dot-notation path to the options key for the current class.
        
        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return 'pane'

    @property
    def background(self) -> Optional[List[PaneBackground]]:
        """An array of background items for the pane.

        :rtype: :class:`list <python:list>` of :class:`PaneBackground`, or
          :obj:`None <python:None>`
        """
        return self._background

    @background.setter
    @class_sensitive(PaneBackground, force_iterable = True)
    def background(self, value):
        self._background = value

    @property
    def center(self) -> Optional[List[str | int]]:
        """The center of a polar chart or angular gauge, given as an array of ``[x, y]``
        positions. Positions can be given as integers that transform to pixels, or as
        percentages of the plot area size. Defaults to
        ``[['50%', '50%']]``.

        :returns: Collection of positions, or :obj:`None <python:None>`
        :rtype: :class:`list <python:list>` of two-item :class:`list <python:list>`
          where each item is either a :class:`str <python:str>` or
          :class:`int <python:int>`, or :obj:`None <python:None>`
        """
        return self._center

    @center.setter
    def center(self, value):
        if not value:
            self._center = None
        else:
            value = validators.iterable(value,
                                        allow_empty = False,
                                        minimum_length = 2,
                                        maximum_length = 2)
            validated = []
            for item in value:
                try:
                    item = validators.string(item)
                    if '%' not in item:
                        raise errors.HighchartsValueError('center expects either a '
                                                          'numeric value or a percentage '
                                                          'string. No "%"" symbol found.')
                except ValueError:
                    item = validators.numeric(item)

                validated.append(item)

            self._center = validated

    @property
    def end_angle(self) -> Optional[int | float | Decimal]:
        """The end angle of the polar X axis or gauge value axis, given in degrees where
        ``0`` is north. Defaults to :meth:`Pane.start_angle` if :obj:`None <python:None>`.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._end_angle

    @end_angle.setter
    def end_angle(self, value):
        self._end_angle = validators.numeric(value, allow_empty = True)

    @property
    def inner_size(self) -> Optional[int | float | Decimal | str]:
        """The inner size of the pane, either as a number defining pixels, or a
        percentage defining a percentage of the pane's size. Defaults to
        ``'0%'``.

        :rtype: numeric or :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._inner_size

    @inner_size.setter
    def inner_size(self, value):
        if not value:
            self._inner_size = None
        else:
            try:
                self._inner_size = validators.string(value)
            except ValueError:
                self._inner_size = validators.numeric(value)

    @property
    def size(self) -> Optional[int | float | Decimal | str]:
        """The size of the pane, either as a number defining pixels, or a
        percentage defining a percentage of the pane's size. Defaults to
        ``'85%'``.

        :rtype: numeric or :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._size

    @size.setter
    def size(self, value):
        if not value:
            self._size = None
        else:
            try:
                value = validators.string(value)
                if '%' not in value:
                    raise errors.HighchartsValueError('size expects either a number or '
                                                      'a percentage string. No "%" '
                                                      'symbol found.')
            except (ValueError, TypeError):
                self._size = validators.numeric(value)

    @property
    def start_angle(self) -> Optional[int | float | Decimal]:
        """The start angle of the polar X axis or gauge value axis, given in degrees where
        ``0`` is north. Defaults to ``0``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._start_angle

    @start_angle.setter
    def start_angle(self, value):
        self._start_angle = validators.numeric(value, allow_empty = True)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'background': as_dict.get('background', None),
            'center': as_dict.get('center', None),
            'end_angle': as_dict.get('endAngle', None),
            'inner_size': as_dict.get('innerSize', None),
            'size': as_dict.get('size', None),
            'start_angle': as_dict.get('startAngle', None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'background': self.background,
            'center': self.center,
            'endAngle': self.end_angle,
            'innerSize': self.inner_size,
            'size': self.size,
            'startAngle': self.start_angle
        }

        return untrimmed



"""Tests for ``highcharts.no_data``."""

import pytest

from json.decoder import JSONDecodeError

from highcharts_core.options.plot_options.pictorial import PictorialOptions as cls
from highcharts_core import errors
from tests.fixtures import input_files, check_input_file, to_camelCase, to_js_dict, \
    Class__init__, Class__to_untrimmed_dict, Class_from_dict, Class_to_dict, \
    Class_from_js_literal

STANDARD_PARAMS = [
    ({}, None),
    ({
      'depth': 10,
      'edge_color': '#999',
      'edge_width': 1,
      'group_z_padding': 4
    }, None),
    # + Base Bar Options
    ({
      'depth': 10,
      'edge_color': '#999',
      'edge_width': 1,
      'group_z_padding': 4,
      'grouping': False,
      'group_padding': 6,
      'max_point_width': 12,
      'min_point_length': 12,
      'point_padding': 6,
      'point_range': 24,
      'point_width': 12
    }, None),
]


@pytest.mark.parametrize('kwargs, error', STANDARD_PARAMS)
def test_PictorialOptions__init__(kwargs, error):
    Class__init__(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error', STANDARD_PARAMS)
def test_PictorialOptions__to_untrimmed_dict(kwargs, error):
    Class__to_untrimmed_dict(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error',  STANDARD_PARAMS)
def test_PictorialOptions_from_dict(kwargs, error):
    Class_from_dict(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error',  STANDARD_PARAMS)
def test_PictorialOptions_to_dict(kwargs, error):
    Class_to_dict(cls, kwargs, error)


@pytest.mark.parametrize('filename, as_file, error', [
    ('plot_options/pictorial/03.js', False, None),
    ('plot_options/pictorial/04.js', False, None),

    ('plot_options/pictorial/error-01.js',
     False,
     (errors.HighchartsValueError,
      errors.HighchartsParseError,
      JSONDecodeError,
      TypeError,
      ValueError)),
    ('plot_options/pictorial/error-02.js',
     False,
     (errors.HighchartsValueError,
      errors.HighchartsParseError,
      JSONDecodeError,
      TypeError,
      ValueError)),

    ('plot_options/pictorial/03.js', True, None),
    ('plot_options/pictorial/04.js', True, None),

    ('plot_options/pictorial/error-01.js',
     True,
     (errors.HighchartsValueError,
      errors.HighchartsParseError,
      JSONDecodeError,
      TypeError,
      ValueError)),
    ('plot_options/pictorial/error-02.js',
     True,
     (errors.HighchartsValueError,
      errors.HighchartsParseError,
      JSONDecodeError,
      TypeError,
      ValueError)),

])
def test_PictorialOptions_from_js_literal(input_files, filename, as_file, error):
    Class_from_js_literal(cls, input_files, filename, as_file, error)



from typing import Optional
from decimal import Decimal

from validator_collection import validators

from highcharts_core import errors
from highcharts_core.options.plot_options.pie import PieOptions


class ItemOptions(PieOptions):
    """General options to apply to all Item series types.

    An item chart is an infographic chart where a number of items are laid out in
    either a rectangular or circular pattern. It can be used to visualize counts
    within a group, or for the circular pattern, typically a parliament.

    The circular layout has much in common with a pie chart. Many of the item series
    options, like ``center``, ``size`` and data label positioning, are inherited from
    the :meth:`PlotOptions.pie` series and don't apply for rectangular layouts.

    .. tabs::

      .. tab:: Circular Item Chart

        .. figure:: ../../../_static/item-example-circular.png
          :alt: Circular Item Example Chart
          :align: center

      .. tab:: Rectangular Item Chart

        .. figure:: ../../../_static/item-example-rectangular.png
          :alt: Rectangular Item Example Chart
          :align: center

      .. tab:: Item Chart with Symbols

        .. figure:: ../../../_static/item-example-symbols.png
          :alt: Item Example Chart with Symbols
          :align: center

    """

    def __init__(self, **kwargs):
        self._item_padding = None
        self._layout = None
        self._rows = None

        self.item_padding = kwargs.get('item_padding', None)
        self.layout = kwargs.get('layout', None)
        self.rows = kwargs.get('rows', None)

        super().__init__(**kwargs)

    @property
    def inner_size(self) -> Optional[str | int]:
        """In circular view, the size of the inner diameter of the circle. Defaults to
        ``40%``. Can be a percentage or pixel value.

        .. note::

          Percentages are relative to the outer perimeter. Pixel values are given as
          integers.

        .. warning::

          If :meth:`ItemOptions.rows` is set, this value will be overridden by the
          ``rows`` setting.

        :rtype: :class:`str <python:str>`, :class:`int <python:int>`, or
          :obj:`None <python:None>`
        """
        return self._inner_size

    @inner_size.setter
    def inner_size(self, value):
        if value is None:
            self._inner_size = None
        else:
            try:
                value = validators.string(value)
                if '%' not in value:
                    raise errors.HighchartsValueError('inner_size expects a number or '
                                                      'percentage ("%") string. No "%" '
                                                      'character found.')
            except TypeError:
                value = validators.integer(value, minimum = 0)

            self._inner_size = value

    @property
    def item_padding(self) -> Optional[int | float | Decimal]:
        """The padding between the items, given in relative size where the size of the
        item itself is 1. Defaults to ``0.1``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._item_padding

    @item_padding.setter
    def item_padding(self, value):
        self._item_padding = validators.numeric(value, allow_empty = True)

    @property
    def layout(self) -> Optional[str]:
        """The layout of the items in rectangular view. Defaults to ``'vertical'``.

        Accepts:

          * ``'horizontal'``
          * ``'vertical'``

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._layout

    @layout.setter
    def layout(self, value):
        if not value:
            self._layout = None
        else:
            value = validators.string(value)
            value = value.lower()
            if value not in ['horizontal', 'vertical']:
                raise errors.HighchartsValueError(f'layout expects "horizontal" or '
                                                  f'"vertical", but was: {value}')

            self._layout = value

    @property
    def rows(self) -> Optional[int]:
        """The number of rows to display in the rectangular or circular view. Defaults to
        :obj:`None <python:None>`.

        .. note::

          If the :meth:`ItemOptions.inner_size` is set, it will be overridden by the
          ``rows`` setting.

        :rtype: :class:`int <python:int>` or :obj:`None <python:None>`
        """
        return self._rows

    @rows.setter
    def rows(self, value):
        self._rows = validators.integer(value, allow_empty = True)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'accessibility': as_dict.get('accessibility', None),
            'allow_point_select': as_dict.get('allowPointSelect', None),
            'animation': as_dict.get('animation', None),
            'class_name': as_dict.get('className', None),
            'clip': as_dict.get('clip', None),
            'color': as_dict.get('color', None),
            'cursor': as_dict.get('cursor', None),
            'custom': as_dict.get('custom', None),
            'dash_style': as_dict.get('dashStyle', None),
            'data_labels': as_dict.get('dataLabels', None),
            'description': as_dict.get('description', None),
            'enable_mouse_tracking': as_dict.get('enableMouseTracking', None),
            'events': as_dict.get('events', None),
            'include_in_data_export': as_dict.get('includeInDataExport', None),
            'keys': as_dict.get('keys', None),
            'label': as_dict.get('label', None),
            'legend_symbol': as_dict.get('legendSymbol', None),
            'linked_to': as_dict.get('linkedTo', None),
            'marker': as_dict.get('marker', None),
            'on_point': as_dict.get('onPoint', None),
            'opacity': as_dict.get('opacity', None),
            'point': as_dict.get('point', None),
            'point_description_formatter': as_dict.get('pointDescriptionFormatter', None),
            'selected': as_dict.get('selected', None),
            'show_checkbox': as_dict.get('showCheckbox', None),
            'show_in_legend': as_dict.get('showInLegend', None),
            'skip_keyboard_navigation': as_dict.get('skipKeyboardNavigation', None),
            'sonification': as_dict.get('sonification', None),
            'states': as_dict.get('states', None),
            'sticky_tracking': as_dict.get('stickyTracking', None),
            'threshold': as_dict.get('threshold', None),
            'tooltip': as_dict.get('tooltip', None),
            'turbo_threshold': as_dict.get('turboThreshold', None),
            'visible': as_dict.get('visible', None),

            'border_color': as_dict.get('borderColor', None),
            'border_width': as_dict.get('borderWidth', None),
            'center': as_dict.get('center', None),
            'color_axis': as_dict.get('colorAxis', None),
            'color_index': as_dict.get('colorIndex', None),
            'color_key': as_dict.get('colorKey', None),
            'colors': as_dict.get('colors', None),
            'depth': as_dict.get('depth', None),
            'end_angle': as_dict.get('endAngle', None),
            'fill_color': as_dict.get('fillColor', None),
            'ignore_hidden_point': as_dict.get('ignoreHiddenPoint', None),
            'inner_size': as_dict.get('innerSize', None),
            'linecap': as_dict.get('linecap', None),
            'min_size': as_dict.get('minSize', None),
            'size': as_dict.get('size', None),
            'sliced_offset': as_dict.get('slicedOffset', None),
            'start_angle': as_dict.get('startAngle', None),
            'thickness': as_dict.get('thickness', None),

            'item_padding': as_dict.get('itemPadding', None),
            'layout': as_dict.get('layout', None),
            'rows': as_dict.get('rows', None)
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'itemPadding': self.item_padding,
            'layout': self.layout,
            'rows': self.rows
        }
        parent_as_dict = super()._to_untrimmed_dict(in_cls = in_cls)

        for key in parent_as_dict:
            untrimmed[key] = parent_as_dict[key]

        return untrimmed



from typing import Optional, List
from decimal import Decimal

from validator_collection import validators

from highcharts_core import errors
from highcharts_core.metaclasses import HighchartsMeta
from highcharts_core.decorators import class_sensitive
from highcharts_core.utility_classes.gradients import Gradient
from highcharts_core.utility_classes.patterns import Pattern
from highcharts_core.utility_classes.data_labels import DataLabel
from highcharts_core.utility_functions import validate_color

from highcharts_core.options.axes.x_axis import XAxis


class StackShadow(HighchartsMeta):
    """Configures the background of stacked points.
    
    .. versionadded:: Highcharts Core (JS) v.11.0.0
    
    .. note::
        
      Only applies to the :class:`PictorialSeries <highcharts_core.options.series.pictorial.PictorialSeries>`.
        
    """
    
    def __init__(self, **kwargs):
        self._border_color = None
        self._border_width = None
        self._color = None
        self._enabled = None
        
        self.border_color = kwargs.get('border_color', None)
        self.border_width = kwargs.get('border_width', None)
        self.color = kwargs.get('color', None)
        self.enabled = kwargs.get('enabled', None)
        
    @property
    def _dot_path(self) -> Optional[str]:
        """The dot-notation path to the options key for the current class.
        
        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return 'yAxis.stackShadow'

    @property
    def border_color(self) -> Optional[str | Gradient | Pattern]:
        """The color of the stack shadow border. Defaults to ``'transparent'``.

        :rtype: :class:`str <python:str>` or
          :class:`Gradient <highcharts_maps.utility_classes.gradients.Gradient>` or
          :class:`Pattern <highcharts_maps.utility_classes.patterns.Pattern>` or
          :obj:`None <python:None>`
        """
        return self._border_color

    @border_color.setter
    def border_color(self, value):
        self._border_color = validate_color(value)

    @property
    def border_width(self) -> Optional[int | float | Decimal]:
        """The width of the border surrounding the stack shadow. Defaults to ``0``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._border_width

    @border_width.setter
    def border_width(self, value):
        self._border_width = validators.numeric(value,
                                                allow_empty = True,
                                                minimum = 0)

    @property
    def color(self) -> Optional[str | Gradient | Pattern]:
        """The color of the stack shadow. Defaults to ``#dedede``.

        :rtype: :class:`str <python:str>`, :class:`Gradient`, :class:`Pattern``, or
          :obj:`None <python:None>`

        """
        return self._color

    @color.setter
    def color(self, value):
        self._color = validate_color(value)

    @property
    def enabled(self) -> Optional[bool]:
        """If ``True``, enable the stack shadow. Defaults to :obj:`None <python:None>`.
        
        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._enabled
    
    @enabled.setter
    def enabled(self, value):
        if value is None:
            self._enabled = None
        else:
            self._enabled = bool(value)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'border_color': as_dict.get('borderColor', None),
            'border_width': as_dict.get('borderWidth', None),
            'color': as_dict.get('color', None),
            'enabled': as_dict.get('enabled', None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'borderColor': self.border_color,
            'borderWidth': self.border_width,
            'color': self.color,
            'enabled': self.enabled,
        }

        return untrimmed


class YAxis(XAxis):
    """Configuration settings for the Y axis or value axis.

    Normally, this is the vertical axis, though if the chart is inverted this becomes
    the horizontal axis."""

    def __init__(self, **kwargs):
        self._max_color = None
        self._min_color = None
        self._stack_labels = None
        self._stack_shadow = None
        self._stops = None
        self._tooltip_value_format = None

        self.max_color = kwargs.get('max_color', None)
        self.min_color = kwargs.get('min_color', None)
        self.stack_labels = kwargs.get('stack_labels', None)
        self.stack_shadow = kwargs.get('stack_shadow', None)
        self.stops = kwargs.get('stops', None)
        self.tooltip_value_format = kwargs.get('tooltip_value_format', None)

        super().__init__(**kwargs)

    @property
    def _dot_path(self) -> Optional[str]:
        """The dot-notation path to the options key for the current class.
        
        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return 'yAxis'

    @property
    def max_color(self) -> Optional[str | Gradient | Pattern]:
        """The color used to represent the maximum value of the Y-axis. Defaults to
        ``'#003399'``.

        .. warning::

          Only applies to :term:`gauge charts <Gauge Chart>`.

        .. warning::

          Will be ignored if :meth:`YAxis.stops` are set.

        :rtype: :class:`str <python:str>`, :class:`Gradient`, :class:`Pattern``, or
          :obj:`None <python:None>`
        """
        return self._max_color

    @max_color.setter
    def max_color(self, value):
        from highcharts_core import utility_functions
        self._max_color = utility_functions.validate_color(value)

    @property
    def min_color(self) -> Optional[str | Gradient | Pattern]:
        """The color used to represent the minimum value of the Y-axis. Defaults to
        ``'#e6bef5'``.

        .. warning::

          Only applies to :term:`gauge charts <Gauge Chart>`.

        .. warning::

          Will be ignored if :meth:`YAxis.stops` are set.

        :rtype: :class:`str <python:str>`, :class:`Gradient`, :class:`Pattern``, or
          :obj:`None <python:None>`
        """
        return self._min_color

    @min_color.setter
    def min_color(self, value):
        from highcharts_core import utility_functions
        self._min_color = utility_functions.validate_color(value)

    @property
    def stack_labels(self) -> Optional[DataLabel]:
        """Configuration settings for the labels that show the total value for each bar in
        a stacked column or bar chart.

        The label will be placed on top of positive columns and below negative columns.
        In case of an inverted column or bar chart, the label is placed to the right of
        positive bars and to the left of negative bars.

        :rtype: :class:`DataLabel` or :obj:`None <python:None>`
        """
        return self._stack_labels

    @stack_labels.setter
    @class_sensitive(DataLabel)
    def stack_labels(self, value):
        self._stack_labels = value

    @property
    def stack_shadow(self) -> Optional[StackShadow]:
        """Configures the background of stacked points. 
        
        .. versionadded:: Highcharts Core (JS) v.11.0.0
        
        .. note::
          
          Only applies to the :class:`PictorialSeries <highcharts_core.options.series.pictorial.PictorialSeries>`
          
        .. warning::
        
          Requires ``series.stacking`` to be defined.
          
        :rtype: :class:`StackShadow <highcharts_core.options.axes.y_axis.StackShadow>` or :obj:`None <python:None>`
        """
        return self._stack_shadow
    
    @stack_shadow.setter
    @class_sensitive(StackShadow)
    def stack_shadow(self, value):
        self._stack_shadow = value

    @property
    def stops(self) -> Optional[List[List[int | float | Decimal | str]]]:
        """Color stops for use in the gradient of a solid gauge. Defaults to
        :obj:`None <python:None>`.

        .. hint::

          Use this in cases where a linear gradient between a
          :meth:`min_color <YAxis.min_color>` and :meth:`max_color <YAxis.max_color>` is
          not sufficient.

        .. note::

          Expects an iterable of 2-member iterables. Each 2-member iterable should be
          composed of a number and a string. The number should be a value between 0 and
          1 which assigns the relative position of the color in the gradient, while the
          string should represent the color itself.

        :rtype: :class:`list <python:list>` of :class:`list <python:list>`, where each
          second :class:`list <python:list>` consists of a :class:`float <python:float>`
          and a :class:`str <python:str>` / or :obj:`None <python:None>`
        """
        return self._stops

    @stops.setter
    def stops(self, value):
        if not value:
            self._stops = None
        else:
            value = validators.iterable(value)
            processed_items = []
            for item in value:
                item = validators.iterable(item)
                if len(item) != 2:
                    raise errors.HighchartsValueError(f'stops expects a list of 2-'
                                                      f'member iterables. Received a '
                                                      f'{len(item)}-member iterable.')
                item = [validators.float(item[0],
                                         minimum = 0,
                                         maximum = 1),
                        validators.string(item[1])]
                processed_items.append(item)

            self._stops = processed_items

    @property
    def tooltip_value_format(self) -> Optional[str]:
        """Format string that will be used for ``point.y`` and available in (JavaScript)
        ``tooltip.pointFormat`` as ``{point.formattedValue}``. Defaults to
        :obj:`None <python:None>`.

        If not set, the JavaScript ``{point.formattedValue}`` will use other options, in
        this order:

          #. :meth:`YAxis.labels.format <AxisLabelOptions.format>` if set
          #. If :meth:`YAxis.type` is ``'category'``, then category name will be
             displayed.
          #. If :meth:`YAxis.type` is ``'datetime'``, then the value will use the same
             format as :meth:`YAxis.labels`.
          #. If :meth:`YAxis.type` is ``'linear'`` or ``'logarithmic'``, then the simple
             value will be displayed.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`.
        """
        return self._tooltip_value_format

    @tooltip_value_format.setter
    def tooltip_value_format(self, value):
        self._tooltip_value_format = validators.string(value, allow_empty = True)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'accessibility': as_dict.get('accessibility', None),
            'align_ticks': as_dict.get('alignTicks', None),
            'allow_decimals': as_dict.get('allowDecimals', None),
            'alternate_grid_color': as_dict.get('alternateGridColor', None),
            'angle': as_dict.get('angle', None),
            'breaks': as_dict.get('breaks', None),
            'categories': as_dict.get('categories', None),
            'ceiling': as_dict.get('ceiling', None),
            'class_name': as_dict.get('className', None),
            'crossing': as_dict.get('crossing', None),
            'date_time_label_formats': as_dict.get('dateTimeLabelFormats', None),
            'end_on_tick': as_dict.get('endOnTick', None),
            'events': as_dict.get('events', None),
            'floor': as_dict.get('floor', None),
            'grid_line_color': as_dict.get('gridLineColor', None),
            'grid_line_dash_style': as_dict.get('gridLineDashStyle', None),
            'grid_line_interpolation': as_dict.get('gridLineInterpolation', None),
            'grid_line_width': as_dict.get('gridLineWidth', None),
            'grid_z_index': as_dict.get('gridZIndex', None),
            'id': as_dict.get('id', None),
            'labels': as_dict.get('labels', None),
            'linked_to': as_dict.get('linkedTo', None),
            'margin': as_dict.get('margin', None),
            'max': as_dict.get('max', None),
            'max_padding': as_dict.get('maxPadding', None),
            'min': as_dict.get('min', None),
            'minor_grid_line_color': as_dict.get('minorGridLineColor', None),
            'minor_grid_line_dash_style': as_dict.get('minorGridLineDashStyle', None),
            'minor_grid_line_width': as_dict.get('minorGridLineWidth', None),
            'minor_tick_color': as_dict.get('minorTickColor', None),
            'minor_tick_interval': as_dict.get('minorTickInterval', None),
            'minor_tick_length': as_dict.get('minorTickLength', None),
            'minor_tick_position': as_dict.get('minorTickPosition', None),
            'minor_ticks': as_dict.get('minorTicks', None),
            'minor_tick_width': as_dict.get('minorTickWidth', None),
            'min_padding': as_dict.get('minPadding', None),
            'min_range': as_dict.get('minRange', None),
            'min_tick_interval': as_dict.get('minTickInterval', None),
            'offset': as_dict.get('offset', None),
            'opposite': as_dict.get('opposite', None),
            'pane': as_dict.get('pane', None),
            'panning_enabled': as_dict.get('panningEnabled', None),
            'plot_bands': as_dict.get('plotBands', None),
            'plot_lines': as_dict.get('plotLines', None),
            'reversed': as_dict.get('reversed', None),
            'reversed_stacks': as_dict.get('reversedStacks', None),
            'show_first_label': as_dict.get('showFirstLabel', None),
            'show_last_label': as_dict.get('showLastLabel', None),
            'soft_max': as_dict.get('softMax', None),
            'soft_min': as_dict.get('softMin', None),
            'start_of_week': as_dict.get('startOfWeek', None),
            'start_on_tick': as_dict.get('startOnTick', None),
            'tick_amount': as_dict.get('tickAmount', None),
            'tick_color': as_dict.get('tickColor', None),
            'tick_interval': as_dict.get('tickInterval', None),
            'tick_length': as_dict.get('tickLength', None),
            'tickmark_placement': as_dict.get('tickmarkPlacement', None),
            'tick_pixel_interval': as_dict.get('tickPixelInterval', None),
            'tick_position': as_dict.get('tickPosition', None),
            'tick_positioner': as_dict.get('tickPositioner', None),
            'tick_positions': as_dict.get('tickPositions', None),
            'tick_width': as_dict.get('tickWidth', None),
            'title': as_dict.get('title', None),
            'type': as_dict.get('type', None),
            'unique_names': as_dict.get('uniqueNames', None),
            'units': as_dict.get('units', None),
            'visible': as_dict.get('visible', None),
            'z_index': as_dict.get('zIndex', None),
            'zoom_enabled': as_dict.get('zoomEnabled', None),

            'crosshair': as_dict.get('crosshair', None),
            'height': as_dict.get('height', None),
            'left': as_dict.get('left', None),
            'line_color': as_dict.get('lineColor', None),
            'line_width': as_dict.get('lineWidth', None),
            'show_empty': as_dict.get('showEmpty', None),
            'top': as_dict.get('top', None),
            'width': as_dict.get('width', None),

            'max_color': as_dict.get('maxColor', None),
            'min_color': as_dict.get('minColor', None),
            'stack_labels': as_dict.get('stackLabels', None),
            'stack_shadow': as_dict.get('stackShadow', None),
            'stops': as_dict.get('stops', None),
            'tooltip_value_format': as_dict.get('tooltipValueFormat', None)
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'maxColor': self.max_color,
            'minColor': self.min_color,
            'stackLabels': self.stack_labels,
            'stackShadow': self.stack_shadow,
            'stops': self.stops,
            'tooltipValueFormat': self.tooltip_value_format
        }

        parent_as_dict = super()._to_untrimmed_dict(in_cls = in_cls)
        for key in parent_as_dict:
            untrimmed[key] = parent_as_dict[key]

        return untrimmed



from typing import Optional, List
from decimal import Decimal

from validator_collection import validators

from highcharts_core import constants, errors, utility_functions
from highcharts_core.decorators import class_sensitive, validate_types
from highcharts_core.options.plot_options.generic import GenericTypeOptions
from highcharts_core.utility_classes.gradients import Gradient
from highcharts_core.utility_classes.patterns import Pattern
from highcharts_core.options.plot_options.levels import LevelOptions


class DependencyWheelOptions(GenericTypeOptions):
    """General options to apply to all Dependency Wheel series types.

    A dependency wheel chart is a type of flow diagram, where all nodes are laid out
    in a circle, and the flow between the are drawn as link bands.

    .. figure:: ../../../_static/dependencywheel-example.png
      :alt: Dependency Wheel Example Chart
      :align: center

    """

    def __init__(self, **kwargs):
        self._border_color = None
        self._border_width = None
        self._center = None
        self._center_in_category = None
        self._color_by_point = None
        self._color_index = None
        self._colors = None
        self._curve_factor = None
        self._levels = None
        self._link_opacity = None
        self._min_link_width = None
        self._node_padding = None
        self._node_width = None
        self._start_angle = None

        self.border_color = kwargs.get('border_color', None)
        self.border_width = kwargs.get('border_width', None)
        self.center = kwargs.get('center', None)
        self.center_in_category = kwargs.get('center_in_category', None)
        self.color_by_point = kwargs.get('color_by_point', None)
        self.color_index = kwargs.get('color_index', None)
        self.colors = kwargs.get('colors', None)
        self.curve_factor = kwargs.get('curve_factor', None)
        self.levels = kwargs.get('levels', None)
        self.link_opacity = kwargs.get('link_opacity', None)
        self.min_link_width = kwargs.get('min_link_width', None)
        self.node_padding = kwargs.get('node_padding', None)
        self.node_width = kwargs.get('node_width', None)
        self.start_angle = kwargs.get('start_angle', None)

        super().__init__(**kwargs)

    @property
    def border_color(self) -> Optional[str | Gradient | Pattern]:
        """The color of the border surrounding each column or bar. Defaults to
        ``'#ffffff'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._border_color

    @border_color.setter
    def border_color(self, value):
        from highcharts_core import utility_functions
        self._border_color = utility_functions.validate_color(value)

    @property
    def border_width(self) -> Optional[int | float | Decimal]:
        """The width of the border surrounding each column or bar. If
        :obj:`None <python:None>`, defaults to ``1`` when there is room for a border, but
        to ``0`` when the columns are so dense that a border would cover the next column.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._border_width

    @border_width.setter
    def border_width(self, value):
        self._border_width = validators.numeric(value,
                                                allow_empty = True,
                                                minimum = 0)

    @property
    def center(self) -> Optional[List[str | int | float | Decimal | constants.EnforcedNullType]]:
        """The center of the wheel relative to the plot area. Expects a two-member array,
        indicating the x and y coordinates as percentages or pixel values. If
        :obj:`None <python:None>`, the default behaviour is to center the wheel inside the
        plot area.

        :rtype: :obj:`None <python:None>` or :class:`list <python:list>` of numeric or
          :class:`str <python:str>` values
        """
        return self._center

    @center.setter
    def center(self, value):
        if not value:
            self._center = None
        else:
            value = validators.iterable(value)
            if len(value) != 2:
                raise errors.HighchartsValueError(f'center expects a 2-member array. '
                                                  f'Received a {len(value)}-member '
                                                  f'array.')
            processed_values = []
            for item in value:
                try:
                    item = validators.string(item)
                    if '%' not in item:
                        raise errors.HighchartsValueError('center expects a 2-member '
                                                          'array of numbers or '
                                                          'percentage strings. No "%" '
                                                          'was found in the string.')
                except TypeError:
                    item = validators.numeric(value)

                processed_values.append(item)

            self._center = processed_values

    @property
    def center_in_category(self) -> Optional[bool]:
        """If ``True``, the columns will center in the category, ignoring null or missing
        points. When ``False``, space will be reserved for null or missing points.
        Defaults to ``False``.

        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._center_in_category

    @center_in_category.setter
    def center_in_category(self, value):
        if value is None:
            self._center_in_category = None
        else:
            self._center_in_category = bool(value)

    @property
    def color_by_point(self) -> Optional[bool]:
        """When using automatic point colors pulled from the global colors or
        series-specific collections, this option determines whether the chart should
        receive one color per series (``False``) or one color per point (``True``).

        Defaults to ``True``.

        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._color_by_point

    @color_by_point.setter
    def color_by_point(self, value):
        if value is None:
            self._color_by_point = None
        else:
            self._color_by_point = bool(value)

    @property
    def color_index(self) -> Optional[int]:
        """When operating in :term:`styled mode`, a specific color index to use for the
        series, so that its graphic representations are given the class name
        ``highcharts-color-{n}``.

        Defaults to :obj:`None <python:None>`.

        :rtype: :class:`int <python:int>` or :obj:`None <python:None>`
        """
        return self._color_index

    @color_index.setter
    def color_index(self, value):
        self._color_index = validators.integer(value,
                                               allow_empty = True,
                                               minimum = 0)

    @property
    def colors(self) -> Optional[List[str | Gradient | Pattern]]:
        """A series-specific or series type-specific color set to apply instead of the
        global colors when :meth:`ArcDiagramOptions.color_by_point` is ``True``.

        :rtype: :class:`list <python:list>` of :class:`str <python:str>`,
          :class:`Gradient`, or :class:`Pattern` OR :obj:`None <python:None>`
        """
        return self._colors

    @colors.setter
    def colors(self, value):
        if not value:
            self._colors = None
        else:
            value = validators.iterable(value)
            self._colors = [utility_functions.validate_color(x) for x in value]

    @property
    def curve_factor(self) -> Optional[int | float | Decimal]:
        """The amount by which to curve the lines in a sankey diagram or dependency wheel.
        Higher numbers makes the links more curved, while a curveFactor of ``0`` makes the
        lines straight. Defaults to ``0.6``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._curve_factor

    @curve_factor.setter
    def curve_factor(self, value):
        self._curve_factor = validators.numeric(value, allow_empty = True)

    @property
    def levels(self) -> Optional[List[LevelOptions]]:
        """Set options on specific levels. Takes precedence over series options, but not
        node and link options.

        :rtype: :obj:`None <python:None>`, or :class:`list <python:list>` of
          :class:`LevelOptions`
        """
        return self._levels

    @levels.setter
    @class_sensitive(LevelOptions, force_iterable = True)
    def levels(self, value):
        self._levels = value

    @property
    def link_opacity(self) -> Optional[int | float | Decimal]:
        """Opacity for the links between nodes in sankey or similar diagrams. Defaults to
        ``0.5``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._link_opacity

    @link_opacity.setter
    def link_opacity(self, value):
        self._link_opacity = validators.numeric(value,
                                                allow_empty = True,
                                                minimum = 0)

    @property
    def min_link_width(self) -> Optional[int | float | Decimal]:
        """The minimal width for a line of a sankey or similar diagram. By default,
        ``0`` values are not shown. Defaults to ``0``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._min_link_width

    @min_link_width.setter
    def min_link_width(self, value):
        self._min_link_width = validators.numeric(value,
                                                  allow_empty = True,
                                                  minimum = 0)

    @property
    def node_padding(self) -> Optional[int | float | Decimal]:
        """The padding between nodes in a sankey diagram or dependency wheel, in pixels.
        Defaults to ``10``.

        .. note::

          If the number of nodes is so great that it is not possible to lay them out
          within the plot area with the given ``node_padding``, they will be rendered with
          a smaller padding as a strategy to avoid overflow.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._node_padding

    @node_padding.setter
    def node_padding(self, value):
        self._node_padding = validators.numeric(value,
                                                allow_empty = True)

    @property
    def node_width(self) -> Optional[int | float | Decimal]:
        """The pixel width of each node in a sankey diagram or dependency wheel, or the
        height in case the chart is inverted. Defaults to ``20``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._node_width

    @node_width.setter
    def node_width(self, value):
        self._node_width = validators.numeric(value,
                                              allow_empty = True,
                                              minimum = 0)

    @property
    def start_angle(self) -> Optional[int | float | Decimal]:
        """The start angle of the dependency wheel, in degrees where ``0`` is up. Defaults
        to ``0``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._start_angle

    @start_angle.setter
    def start_angle(self, value):
        self._start_angle = validators.numeric(value,
                                               allow_empty = True,
                                               minimum = 0,
                                               maximum = 360)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'accessibility': as_dict.get('accessibility', None),
            'allow_point_select': as_dict.get('allowPointSelect', None),
            'animation': as_dict.get('animation', None),
            'class_name': as_dict.get('className', None),
            'clip': as_dict.get('clip', None),
            'color': as_dict.get('color', None),
            'cursor': as_dict.get('cursor', None),
            'custom': as_dict.get('custom', None),
            'dash_style': as_dict.get('dashStyle', None),
            'data_labels': as_dict.get('dataLabels', None),
            'description': as_dict.get('description', None),
            'enable_mouse_tracking': as_dict.get('enableMouseTracking', None),
            'events': as_dict.get('events', None),
            'include_in_data_export': as_dict.get('includeInDataExport', None),
            'keys': as_dict.get('keys', None),
            'label': as_dict.get('label', None),
            'legend_symbol': as_dict.get('legendSymbol', None),
            'linked_to': as_dict.get('linkedTo', None),
            'marker': as_dict.get('marker', None),
            'on_point': as_dict.get('onPoint', None),
            'opacity': as_dict.get('opacity', None),
            'point': as_dict.get('point', None),
            'point_description_formatter': as_dict.get('pointDescriptionFormatter', None),
            'selected': as_dict.get('selected', None),
            'show_checkbox': as_dict.get('showCheckbox', None),
            'show_in_legend': as_dict.get('showInLegend', None),
            'skip_keyboard_navigation': as_dict.get('skipKeyboardNavigation', None),
            'sonification': as_dict.get('sonification', None),
            'states': as_dict.get('states', None),
            'sticky_tracking': as_dict.get('stickyTracking', None),
            'threshold': as_dict.get('threshold', None),
            'tooltip': as_dict.get('tooltip', None),
            'turbo_threshold': as_dict.get('turboThreshold', None),
            'visible': as_dict.get('visible', None),

            'border_color': as_dict.get('borderColor', None),
            'border_width': as_dict.get('borderWidth', None),
            'center': as_dict.get('center', None),
            'center_in_category': as_dict.get('centerInCategory', None),
            'color_by_point': as_dict.get('colorByPoint', None),
            'color_index': as_dict.get('colorIndex', None),
            'colors': as_dict.get('colors', None),
            'curve_factor': as_dict.get('curveFactor', None),
            'levels': as_dict.get('levels', None),
            'link_opacity': as_dict.get('linkOpacity', None),
            'min_link_width': as_dict.get('minLinkWidth', None),
            'node_padding': as_dict.get('nodePadding', None),
            'node_width': as_dict.get('nodeWidth', None),
            'start_angle': as_dict.get('startAngle', None)
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'borderColor': self.border_color,
            'borderWidth': self.border_width,
            'center': self.center,
            'centerInCategory': self.center_in_category,
            'colorByPoint': self.color_by_point,
            'colorIndex': self.color_index,
            'colors': self.colors,
            'curveFactor': self.curve_factor,
            'levels': self.levels,
            'linkOpacity': self.link_opacity,
            'minLinkWidth': self.min_link_width,
            'nodePadding': self.node_padding,
            'nodeWidth': self.node_width,
            'startAngle': self.start_angle
        }
        parent_as_dict = super()._to_untrimmed_dict(in_cls = in_cls)

        for key in parent_as_dict:
            untrimmed[key] = parent_as_dict[key]

        return untrimmed



from typing import Optional, List

from validator_collection import validators, checkers

from highcharts_core import constants, errors
from highcharts_core.decorators import class_sensitive
from highcharts_core.metaclasses import HighchartsMeta
from highcharts_core.global_options.language.accessibility import AccessibilityLanguageOptions
from highcharts_core.global_options.language.export_data import ExportDataLanguageOptions
from highcharts_core.global_options.language.navigation import NavigationLanguageOptions


class Language(HighchartsMeta):
    """Collection of configuration settings for UI strings that can be adapted for
    display in specific languages.

    .. note::

      The :class:`Language` object is a global setting in Highcharts and it *cannot* be
      set on each chart initialization. Instead, it has to be set using (in JavaScript)
      ``Highcharts.setOptions(...)`` before any chart is initialized.

    """

    def __init__(self, **kwargs):
        self._accessibility = None
        self._context_button_title = None
        self._decimal_point = None
        self._download_csv = None
        self._download_jpeg = None
        self._download_midi = None
        self._download_pdf = None
        self._download_png = None
        self._download_svg = None
        self._download_xls = None
        self._drillup_text = None
        self._exit_fullscreen = None
        self._export_data = None
        self._hide_data = None
        self._invalid_date = None
        self._loading = None
        self._main_breadcrumb = None
        self._months = None
        self._navigation = None
        self._no_data = None
        self._numeric_symbol_magnitude = None
        self._numeric_symbols = None
        self._play_as_sound = None
        self._print_chart = None
        self._reset_zoom = None
        self._reset_zoom_title = None
        self._short_months = None
        self._short_weekdays = None
        self._thousands_separator = None
        self._view_data = None
        self._view_fullscreen = None
        self._weekdays = None

        self.accessibility = kwargs.get('accessibility', None)
        self.context_button_title = kwargs.get('context_button_title', None)
        self.decimal_point = kwargs.get('decimal_point', None)
        self.download_csv = kwargs.get('download_csv', None)
        self.download_jpeg = kwargs.get('download_jpeg', None)
        self.download_midi = kwargs.get('download_midi', None)
        self.download_pdf = kwargs.get('download_pdf', None)
        self.download_png = kwargs.get('download_png', None)
        self.download_svg = kwargs.get('download_svg', None)
        self.download_xls = kwargs.get('download_xls', None)
        self.drillup_text = kwargs.get('drillup_text', None)
        self.exit_fullscreen = kwargs.get('exit_fullscreen', None)
        self.export_data = kwargs.get('export_data', None)
        self.hide_data = kwargs.get('hide_data', None)
        self.invalid_date = kwargs.get('invalid_date', None)
        self.loading = kwargs.get('loading', None)
        self.main_breadcrumb = kwargs.get('main_breadcrumb', None)
        self.months = kwargs.get('months', None)
        self.navigation = kwargs.get('navigation', None)
        self.no_data = kwargs.get('no_data', None)
        self.numeric_symbol_magnitude = kwargs.get('numeric_symbol_magnitude', None)
        self.numeric_symbols = kwargs.get('numeric_symbols', None)
        self.play_as_sound = kwargs.get('play_as_sound', None)
        self.print_chart = kwargs.get('print_chart', None)
        self.reset_zoom = kwargs.get('reset_zoom', None)
        self.reset_zoom_title = kwargs.get('reset_zoom_title', None)
        self.short_months = kwargs.get('short_months', None)
        self.short_weekdays = kwargs.get('short_weekdays', None)
        self.thousands_separator = kwargs.get('thousands_separator', None)
        self.view_data = kwargs.get('view_data', None)
        self.view_fullscreen = kwargs.get('view_fullscreen', None)
        self.weekdays = kwargs.get('weekdays', None)

    @property
    def _dot_path(self) -> Optional[str]:
        """The dot-notation path to the options key for the current class.
        
        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return 'lang'

    @property
    def accessibility(self) -> Optional[AccessibilityLanguageOptions]:
        """Configuration of accessibility strings in the chart.

        .. note::

          Requires the
          `accessibility module <https://code.highcharts.com/modules/accessibility.js>`_
          to be loaded.

          For a description of the module and information on its features, see
          `Accessibility <https://www.highcharts.com/docs/chart-concepts/accessibility>`_.

        .. hint::

          For more dynamic control over the accessibility functionality, see
          :meth:`Accessibility.point.description_formatter`,
          :meth:`Accessibility.series.description_formatter`, and
          :meth:`Accessibility.screen_reader_section.before_chart_formatter`.

        :returns: Accessibility strings used in the chart.
        :rtype: :class:`AccessibilityLanguageOptions` or :obj:`None <python:None>`
        """
        return self._accessibility

    @accessibility.setter
    @class_sensitive(AccessibilityLanguageOptions)
    def accessibility(self, value):
        self._accessibility = value

    @property
    def context_button_title(self) -> Optional[str]:
        """The tooltip title for the context menu holding print and export menu items.

        Defaults to ``'Chart context menu'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._context_button_title

    @context_button_title.setter
    def context_button_title(self, value):
        self._context_button_title = validators.string(value, allow_empty = True)

    @property
    def decimal_point(self) -> Optional[str]:
        """Decimal point used in (JavaScript) ``Highcharts.numberFormat``. Defaults to
        ``.``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._decimal_point

    @decimal_point.setter
    def decimal_point(self, value):
        self._decimal_point = validators.string(value, allow_empty = True)

    @property
    def download_csv(self) -> Optional[str]:
        """Text for the context menu item that allows the user to download a CSV of the
        chart/data. Defaults to ``'Download CSV'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._download_csv

    @download_csv.setter
    def download_csv(self, value):
        self._download_csv = validators.string(value, allow_empty = True)

    @property
    def download_jpeg(self) -> Optional[str]:
        """Text for the context menu item that allows the user to download a JPEG of the
        chart/data. Defaults to ``'Download JPEG'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._download_jpeg

    @download_jpeg.setter
    def download_jpeg(self, value):
        self._download_jpeg = validators.string(value, allow_empty = True)

    @property
    def download_midi(self) -> Optional[str]:
        """
        .. versionadded:: Highcharts Core for Python v.1.1.0 / Highcharts Core (JS) v.11.0.0
        
          Text for the context menu item that allows the user to download a MIDI of the
          chart/data. Defaults to ``'Download MIDI'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._download_midi

    @download_midi.setter
    def download_midi(self, value):
        self._download_midi = validators.string(value, allow_empty = True)

    @property
    def download_pdf(self) -> Optional[str]:
        """Text for the context menu item that allows the user to download a PDF of the
        chart/data. Defaults to ``'Download PDF'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._download_pdf

    @download_pdf.setter
    def download_pdf(self, value):
        self._download_pdf = validators.string(value, allow_empty = True)

    @property
    def download_png(self) -> Optional[str]:
        """Text for the context menu item that allows the user to download a PNG of the
        chart/data. Defaults to ``'Download PNG'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._download_png

    @download_png.setter
    def download_png(self, value):
        self._download_png = validators.string(value, allow_empty = True)

    @property
    def download_svg(self) -> Optional[str]:
        """Text for the context menu item that allows the user to download an SVG of the
        chart/data. Defaults to ``'Download SVG'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._download_svg

    @download_svg.setter
    def download_svg(self, value):
        self._download_svg = validators.string(value, allow_empty = True)

    @property
    def download_xls(self) -> Optional[str]:
        """Text for the context menu item that allows the user to download a Microsoft
        Excel file  of the chart/data. Defaults to
        ``'Download Excel'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._download_xls

    @download_xls.setter
    def download_xls(self, value):
        self._download_xls = validators.string(value, allow_empty = True)

    @property
    def drillup_text(self) -> Optional[str]:
        """The text for the button that appears when drilling down, linking back to the
        parent series.

        .. note::

          The parent series' name is inserted for ``{series.name}``.

        Defaults to :obj:`None <python:None>`.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._drillup_text

    @drillup_text.setter
    def drillup_text(self, value):
        self._drillup_text = validators.string(value, allow_empty = True)

    @property
    def exit_fullscreen(self) -> Optional[str]:
        """The text for the menu item to exit the chart from full screen. Defaults to
        ``'Exit from full screen'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._exit_fullscreen

    @exit_fullscreen.setter
    def exit_fullscreen(self, value):
        self._exit_fullscreen = validators.string(value, allow_empty = True)

    @property
    def export_data(self) -> Optional[ExportDataLanguageOptions]:
        """Language strings used in the exported data table.

        :rtype: :class:`ExportDataLanguageOptions` or :obj:`None <python:None>`
        """
        return self._export_data

    @export_data.setter
    @class_sensitive(ExportDataLanguageOptions)
    def export_data(self, value):
        self._export_data = value

    @property
    def hide_data(self) -> Optional[str]:
        """Text used for the hide data table menu item. Defaults to:
        ``'Hide data table'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._hide_data

    @hide_data.setter
    def hide_data(self, value):
        self._hide_data = validators.string(value, allow_empty = True)

    @property
    def invalid_date(self) -> Optional[str]:
        """Text to show in a date field for invalid dates. Defaults to
        ``''`` (an empty string).

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._invalid_date

    @invalid_date.setter
    def invalid_date(self, value):
        self._invalid_date = validators.string(value, allow_empty = True)

    @property
    def loading(self) -> Optional[str]:
        """The loading text that appears when the chart is set into the loading state
        following a (JavaScript) call to ``chart.showLoading()``. Defaults to
        ``'Loading...'``.

        :rtype: :class:`str <python:None>`
        """
        return self._loading

    @loading.setter
    def loading(self, value):
        self._loading = validators.string(value, allow_empty = True)

    @property
    def main_breadcrumb(self) -> Optional[str]:
        """The root item in the breadcrums used when in drilldown mode. Defaults to
        ``'Main'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._main_breadcrumb

    @main_breadcrumb.setter
    def main_breadcrumb(self, value):
        self._main_breadcrumb = validators.string(value, allow_empty = True)

    @property
    def months(self) -> Optional[List[str]]:
        """An array containing the months names. Defaults to:

          .. code-block::

            [
                "January",
                "February",
                "March",
                "April",
                "May",
                "June",
                "July",
                "August",
                "September",
                "October",
                "November",
                "December"
            ]

        .. note::

          Corresponds to the ``%B`` format string in (JavaScript)
          ``Highcharts.dateFormat()``.

        :rtype: :class:`list <python:list>` of :class:`str <python:str>`, or
          :obj:`None <python:None>`
        """
        return self._months

    @months.setter
    def months(self, value):
        if not value:
            self._months = None
        else:
            if not checkers.is_iterable(value):
                raise errors.HighchartsValueError(f'months expects an iterable of strings'
                                                  f'. Received a: '
                                                  f'{value.__class__.__name__}')
            elif len(value) != 12:
                raise errors.HighchartsValueError(f'months expects 12 string values, but'
                                                  f'received: {len(value)}')

            self._months = [validators.string(x) for x in value]

    @property
    def navigation(self) -> Optional[NavigationLanguageOptions]:
        """The Popup strings used in the chart.

        .. note::

          Requires the ``annotations.js`` or ``annotations-advanced.src.js`` module to be
          loaded.

        :rtype: :class:`NavigationLanguageOptions` or :obj:`None <python:None>`
        """
        return self._navigation

    @navigation.setter
    @class_sensitive(NavigationLanguageOptions)
    def navigation(self, value):
        self._navigation = value

    @property
    def no_data(self) -> Optional[str]:
        """The text to display when the chart contains no data. Defaults to
        ``'No data to display'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._no_data

    @no_data.setter
    def no_data(self, value):
        self._no_data = validators.string(value, allow_empty = True)

    @property
    def numeric_symbol_magnitude(self) -> Optional[int]:
        """The magnitude of replacements for :meth:`Language.numeric_symbols`
        replacements. Defaults to ``1000``.

        .. hint::

          Use ``10000`` for Japanese, Korean and various Chinese locales, which use
          symbols for 10^4, 10^8 and 10^12.

        :rtype: :class:`int <python:int>`
        """
        return self._numeric_symbol_magnitude

    @numeric_symbol_magnitude.setter
    def numeric_symbol_magnitude(self, value):
        self._numeric_symbol_magnitude = validators.integer(value,
                                                            allow_empty = True,
                                                            minimum = 0,
                                                            coerce_value = True)

    @property
    def numeric_symbols(self) -> Optional[List[str] | constants.EnforcedNullType]:
        """:term:`Metric suffixes <Metric Suffix>` used to shorten high numbers in axis
        labels. Defaults to ``["k", "M", "G", "T", "P", "E"]``.

        .. note::

          Replacing any of the positions with :obj:`None <python:None>` or
          :class:`constants.EnforcedNullType` causes the full number to be written.
          Setting :meth:`Language.numeric_symbols` as a whole to :obj:`None <python:None>`
          or :class:`constants.EnforcedNullType` disables shortening altogether.

        :rtype: :class:`list <python:list>` of :class:`str <python:str>` or
          :class:`EnforcedNullType`, or :class:`EnforcedNullType`, or
          :obj:`None <python:None>`
        """
        return self._numeric_symbols

    @numeric_symbols.setter
    def numeric_symbols(self, value):
        if not value:
            self._numeric_symbols = None
        elif isinstance(value, constants.EnforcedNullType):
            self._numeric_symbols = constants.EnforcedNull
        else:
            if not checkers.is_iterable(value):
                raise errors.HighchartsValueError(f'numeric_symbols expects an iterable '
                                                  f'of values, but received: '
                                                  f'{value.__class__.__name__}')
            validated = []
            for item in value:
                if item is None or item == constants.EnforcedNull:
                    validated.append(item)
                else:
                    validated.append(validators.string(item))

            self._numeric_symbols = validated

    @property
    def play_as_sound(self) -> Optional[str]:
        """
        .. versionadded:: Highcharts Core for Python v.1.1.0 / Highcharts Core (JS) v.11.0.0

          Text for the context menu item that allows the user to play the chart/data as a sound. 
          Defaults to ``'Play as sound'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._play_as_sound

    @play_as_sound.setter
    def play_as_sound(self, value):
        self._play_as_sound = validators.string(value, allow_empty = True)

    @property
    def print_chart(self) -> Optional[str]:
        """The text for the menu item to print the chart. Defaults to
        ``'Print chart'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._print_chart

    @print_chart.setter
    def print_chart(self, value):
        self._print_chart = validators.string(value, allow_empty = True)

    @property
    def reset_zoom(self) -> Optional[str]:
        """The text for the label of the button to reset the zoom when a chart is zoomed.
        Defaults to ``'Reset zoom'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._reset_zoom

    @reset_zoom.setter
    def reset_zoom(self, value):
        self._reset_zoom = validators.string(value, allow_empty = True)

    @property
    def reset_zoom_title(self) -> Optional[str]:
        """The text for the tooltip which appears above the button to reset the zoom when
        a chart is zoomed. Defaults to ``'Reset zoom level 1:1'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._reset_zoom_title

    @reset_zoom_title.setter
    def reset_zoom_title(self, value):
        self._reset_zoom_title = validators.string(value, allow_empty = True)

    @property
    def short_months(self) -> Optional[List[str]]:
        """A collection containing the months names in abbreviated form. Defaults to:

          .. code-block::

            [
              "Jan",
              "Feb",
              "Mar",
              "Apr",
              "May",
              "Jun",
              "Jul",
              "Aug",
              "Sep",
              "Oct",
              "Nov",
              "Dec"
            ]

        .. note::

          Corresponds to the ``%b`` format in (JavaScript) ``Highcharts.dateFormat()``.

        :rtype: :class:`list <python:list>` of :class:`str <python:str>`, or
          :obj:`None <python:None>`
        """
        return self._short_months

    @short_months.setter
    def short_months(self, value):
        if not value:
            self._short_months = None
        else:
            if not checkers.is_iterable(value):
                raise errors.HighchartsValueError(f'short_months expects an iterable of '
                                                  f'strings. Received a: '
                                                  f'{value.__class__.__name__}')
            elif len(value) != 12:
                raise errors.HighchartsValueError(f'short_months expects 12 string values'
                                                  f', but received: {len(value)}')

            self._short_months = [validators.string(x) for x in value]

    @property
    def short_weekdays(self) -> Optional[List[str]]:
        """Short week days, starting Sunday.

        If not specified, Highcharts uses the first three letters of the
        :meth:`Language.weekdays` setting.

        :rtype: :class:`list <python:list>` of :class:`str <python:str>` or
          :obj:`None <python:None>`
        """
        return self._short_weekdays

    @short_weekdays.setter
    def short_weekdays(self, value):
        if not value:
            self._short_weekdays = None
        else:
            if not checkers.is_iterable(value):
                raise errors.HighchartsValueError(f'short_weekdays expects an iterable. '
                                                  f'Received: {value.__class__.__name__}')

            self._short_weekdays = [validators.string(x) for x in value]

    @property
    def thousands_separator(self) -> Optional[str]:
        """The default thousands separator used in the (JavaScript)
        ``Highcharts.numberFormat()`` method unless otherwise specified in the function
        arguments.

        Defaults to a single space character, which is recommended in ISO 31-0 and works
        across Anglo-American and continental European languages.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._thousands_separator

    @thousands_separator.setter
    def thousands_separator(self, value):
        self._thousands_separator = validators.string(value, allow_empty = True)

    @property
    def view_data(self) -> Optional[str]:
        """The text for the menu item to view the chart's data table. Defaults to:
        ``'View data table'``

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._view_data

    @view_data.setter
    def view_data(self, value):
        self._view_data = validators.string(value, allow_empty = True)

    @property
    def view_fullscreen(self) -> Optional[str]:
        """The text for the menu item to view the chart in fullscreen mode. Defaults to:
        ``'View in full screen'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._view_fullscreen

    @view_fullscreen.setter
    def view_fullscreen(self, value):
        self._view_fullscreen = validators.string(value, allow_empty = True)

    @property
    def weekdays(self) -> Optional[List[str]]:
        """An array containing the days of the week, starting with Sunday. Defaults to:

        .. code-block::

          [
            "Sunday",
            "Monday",
            "Tuesday",
            "Wednesday",
            "Thursday",
            "Friday",
            "Saturday"
          ]

        :rtype: :class:`list <python:list>` of :class:`str <python:str>`, or
          :obj:`None <python:None>`
        """
        return self._weekdays

    @weekdays.setter
    def weekdays(self, value):
        if not value:
            self._weekdays = None
        else:
            if not checkers.is_iterable(value):
                raise errors.HighchartsValueError(f'weekdays expects an iterable of '
                                                  f'strings. Received a: '
                                                  f'{value.__class__.__name__}')
            elif len(value) != 7:
                raise errors.HighchartsValueError(f'weekdays expects 7 string values, but'
                                                  f'received: {len(value)}')

            self._weekdays = [validators.string(x) for x in value]

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'accessibility': as_dict.get('accessibility', None),
            'context_button_title': as_dict.get('contextButtonTitle', None),
            'decimal_point': as_dict.get('decimalPoint', None),
            'download_csv': as_dict.get('downloadCSV', None),
            'download_jpeg': as_dict.get('downloadJPEG', None),
            'download_midi': as_dict.get('downloadMIDI', None),
            'download_pdf': as_dict.get('downloadPDF', None),
            'download_png': as_dict.get('downloadPNG', None),
            'download_svg': as_dict.get('downloadSVG', None),
            'download_xls': as_dict.get('downloadXLS', None),
            'drillup_text': as_dict.get('drillUpText', None),
            'exit_fullscreen': as_dict.get('exitFullscreen', None),
            'export_data': as_dict.get('exportData', None),
            'hide_data': as_dict.get('hideData', None),
            'invalid_date': as_dict.get('invalidDate', None),
            'loading': as_dict.get('loading', None),
            'main_breadcrumb': as_dict.get('mainBreadcrumb', None),
            'months': as_dict.get('months', None),
            'navigation': as_dict.get('navigation', None),
            'no_data': as_dict.get('noData', None),
            'numeric_symbol_magnitude': as_dict.get('numericSymbolMagnitude', None),
            'numeric_symbols': as_dict.get('numericSymbols', None),
            'play_as_sound': as_dict.get('playAsSound', None),
            'print_chart': as_dict.get('printChart', None),
            'reset_zoom': as_dict.get('resetZoom', None),
            'reset_zoom_title': as_dict.get('resetZoomTitle', None),
            'short_months': as_dict.get('shortMonths', None),
            'short_weekdays': as_dict.get('shortWeekdays', None),
            'thousands_separator': as_dict.get('thousandsSep', None),
            'view_data': as_dict.get('viewData', None),
            'view_fullscreen': as_dict.get('viewFullscreen', None),
            'weekdays': as_dict.get('weekdays',  None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'accessibility': self.accessibility,
            'contextButtonTitle': self.context_button_title,
            'decimalPoint': self.decimal_point,
            'downloadCSV': self.download_csv,
            'downloadJPEG': self.download_jpeg,
            'downloadMIDI': self.download_midi,
            'downloadPDF': self.download_pdf,
            'downloadPNG': self.download_png,
            'downloadSVG': self.download_svg,
            'downloadXLS': self.download_xls,
            'drillUpText': self.drillup_text,
            'exitFullscreen': self.exit_fullscreen,
            'exportData': self.export_data,
            'hideData': self.hide_data,
            'invalidDate': self.invalid_date,
            'loading': self.loading,
            'mainBreadcrumb': self.main_breadcrumb,
            'months': self.months,
            'navigation': self.navigation,
            'noData': self.no_data,
            'numericSymbolMagnitude': self.numeric_symbol_magnitude,
            'numericSymbols': self.numeric_symbols,
            'playAsSound': self.play_as_sound,
            'printChart': self.print_chart,
            'resetZoom': self.reset_zoom,
            'resetZoomTitle': self.reset_zoom_title,
            'shortMonths': self.short_months,
            'shortWeekdays': self.short_weekdays,
            'thousandsSep': self.thousands_separator,
            'viewData': self.view_data,
            'viewFullscreen': self.view_fullscreen,
            'weekdays': self.weekdays
        }

        return untrimmed


__all__ = [
    'Language',
    'AccessibilityLanguageOptions',
    'ExportDataLanguageOptions',
    'NavigationLanguageOptions'
]



from typing import Optional
from decimal import Decimal

from validator_collection import validators

from highcharts_core import errors
from highcharts_core.metaclasses import HighchartsMeta


class Position(HighchartsMeta):
    """The position of the Reset Zoom Button."""

    def __init__(self, **kwargs):
        self._align = None
        self._vertical_align = None
        self._x = None
        self._y = None

        self.align = kwargs.get('align', None)
        self.vertical_align = kwargs.get('vertical_align', None)
        self.x = kwargs.get('x', None)
        self.y = kwargs.get('y', None)

    @property
    def align(self) -> Optional[str]:
        """The horizontal alignment of the button. Defaults to ``'right'``.

        Accepts:

          * ``'left'``
          * ``'center'``
          * ``'right'``

        :returns: The horizontal alignment of the button.
        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._align

    @align.setter
    def align(self, value):
        if not value:
            self._align = None
        else:
            value = validators.string(value, allow_empty = False)
            value = value.lower()
            if value not in ['left', 'center', 'right']:
                raise errors.HighchartsValueError(f'align must be either "left", '
                                                  f'"center", or "right". Was: {value}')

        self._align = value

    @property
    def vertical_align(self) -> Optional[str]:
        """The vertical alignment of the button. Defaults to ``'top'``.

        Accepts:

          * ``'bottom'``
          * ``'middle'``
          * ``'top'``

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._vertical_align

    @vertical_align.setter
    def vertical_align(self, value):
        if not value:
            self._vertical_align = None
        else:
            value = validators.string(value, allow_empty = True)
            value = value.lower()
            if value not in ['bottom', 'middle', 'top']:
                raise errors.HighchartsValueError(f'vertical_align expects either "top", '
                                                  f'"middle", or "bottom". Was: {value}')
            self._vertical_align = value

    @property
    def x(self) -> Optional[int | float | Decimal]:
        """The x position offset of the button. Defaults to ``-10``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._x

    @x.setter
    def x(self, value):
        self._x = validators.numeric(value, allow_empty = True)

    @property
    def y(self) -> Optional[int | float | Decimal]:
        """The y position offset of the button. Defaults to ``10``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._y

    @y.setter
    def y(self, value):
        self._y = validators.numeric(value, allow_empty = True)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'align': as_dict.get('align', None),
            'vertical_align': as_dict.get('verticalAlign', None),
            'x': as_dict.get('x', None),
            'y': as_dict.get('y', None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        return {
            'align': self.align,
            'verticalAlign': self.vertical_align,
            'x': self.x,
            'y': self.y
        }



from typing import Optional
from decimal import Decimal

from validator_collection import validators

from highcharts_core import errors
from highcharts_core.decorators import class_sensitive, validate_types
from highcharts_core.metaclasses import HighchartsMeta
from highcharts_core.utility_classes.animation import AnimationOptions
from highcharts_core.utility_classes.gradients import Gradient
from highcharts_core.utility_classes.patterns import Pattern


class HoverState(HighchartsMeta):
    """Options for the hovered point/series."""

    def __init__(self, **kwargs):
        self._animation = None
        self._border_color = None
        self._brightness = None
        self._color = None
        self._enabled = None

        self.animation = kwargs.get('animation', None)
        self.border_color = kwargs.get('border_color', None)
        self.brightness = kwargs.get('brightness', None)
        self.color = kwargs.get('color', None)
        self.enabled = kwargs.get('enabled', None)

    @property
    def animation(self) -> Optional[AnimationOptions]:
        """Animation setting for hovering the graph in line-type series.

        :rtype: :class:`AnimationOptions` or :obj:`None <python:None>`
        """
        return self._animation

    @animation.setter
    @class_sensitive(AnimationOptions)
    def animation(self, value):
        self._animation = value

    @property
    def border_color(self) -> Optional[str | Gradient | Pattern]:
        """A specific border color for the hovered point. If :obj:`None <python:None>`,
        defaults to inherit the normal state border color.

        :returns: The border color for the hovered point.
        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._border_color

    @border_color.setter
    def border_color(self, value):
        from highcharts_core import utility_functions
        self._border_color = utility_functions.validate_color(value)

    @property
    def brightness(self) -> Optional[int | float | Decimal]:
        """How much to brighten the point on interaction. Defaults to ``0.1``.

        Requires the main color to be defined in hex or rgb(a) format.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._brightness

    @brightness.setter
    def brightness(self, value):
        self._brightness = validators.numeric(value, allow_empty = True)

    @property
    def color(self) -> Optional[str | Gradient | Pattern]:
        """A specific color for the hovered point.

        :rtype: :class:`str <python:str>`, :class:`Gradient`, :class:`Pattern`, or
          :obj:`None <python:None>`
        """
        return self._color

    @color.setter
    def color(self, value):
        from highcharts_core import utility_functions
        self._color = utility_functions.validate_color(value)

    @property
    def color(self) -> Optional[str]:
        """A specific color for the hovered point.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._color

    @color.setter
    def color(self, value):
        self._color = validators.string(value, allow_empty = True)

    @property
    def enabled(self) -> Optional[bool]:
        """Enable separate styles for the hovered series to visualize that the user hovers
        either the series itself or the legend. Defaults to ``True``.

        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._enabled

    @enabled.setter
    def enabled(self, value):
        if value is None:
            self._enabled = None
        else:
            self._enabled = bool(value)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'animation': as_dict.get('animation', None),
            'border_color': as_dict.get('borderColor', None),
            'brightness': as_dict.get('brightness', None),
            'color': as_dict.get('color', None),
            'enabled': as_dict.get('enabled', None)
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'animation': self.animation,
            'borderColor': self.border_color,
            'brightness': self.brightness,
            'color': self.color,
            'enabled': self.enabled
        }

        return untrimmed


class InactiveState(HighchartsMeta):
    """Options for the oppositive of a hovered point/series."""

    def __init__(self, **kwargs):
        self._animation = None
        self._enabled = None
        self._opacity = None

        self.animation = kwargs.get('animation', None)
        self.enabled = kwargs.get('enabled', None)
        self.opacity = kwargs.get('opacity', None)

    @property
    def animation(self) -> Optional[AnimationOptions]:
        """Animation setting when not hovering over the marker.

        :rtype: :class:`AnimationOptions` or :obj:`None <python:None>`
        """
        return self._animation

    @animation.setter
    @class_sensitive(AnimationOptions)
    def animation(self, value):
        self._animation = value

    @property
    def enabled(self) -> Optional[bool]:
        """Enable or disable the inactive state for the series. Defaults to ``True``.

        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._enabled

    @enabled.setter
    def enabled(self, value):
        if value is None:
            self._enabled = None
        else:
            self._enabled = bool(value)

    @property
    def opacity(self) -> Optional[int | float | Decimal]:
        """Opacity of series elements (dataLabels, line, area). Defaults to ``0.2``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._opacity

    @opacity.setter
    def opacity(self, value):
        self._opacity = validators.numeric(value, allow_empty = True)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'animation': as_dict.get('animation', None),
            'enabled': as_dict.get('enabled', None),
            'opacity': as_dict.get('opacity', None)
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'animation': self.animation,
            'enabled': self.enabled,
            'opacity': self.opacity
        }

        return untrimmed


class NormalState(HighchartsMeta):
    """Options for returning to a normal state after hovering"""

    def __init__(self, **kwargs):
        self._animation = None

        self.animation = kwargs.get('animation', None)

    @property
    def animation(self) -> Optional[bool | AnimationOptions]:
        """Animation setting when returning to a normal state after hovering. Defaults to
        ``True``.

        :rtype: :class:`bool <python:bool>` or :class:`AnimationOptions` or
          :obj:`None <python:None>`
        """
        return self._animation

    @animation.setter
    def animation(self, value):
        if isinstance(value, bool):
            self._animation = value
        else:
            self._animation = validate_types(value,
                                             types = AnimationOptions)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'animation': as_dict.get('animation', None)
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'animation': self.animation
        }

        return untrimmed


class SelectState(HighchartsMeta):
    """Options for the selected point. These settings override the normal state options
    when a point is selected."""

    def __init__(self, **kwargs):
        self._animation = None
        self._border_color = None
        self._color = None
        self._enabled = None

        self.animation = kwargs.get('animation', None)
        self.border_color = kwargs.get('border_color', None)
        self.color = kwargs.get('color', None)
        self.enabled = kwargs.get('enabled', None)

    @property
    def animation(self) -> Optional[AnimationOptions]:
        """Animation setting for hovering the graph in line-type series.

        :rtype: :class:`AnimationOptions` or :obj:`None <python:None>`
        """
        return self._animation

    @animation.setter
    @class_sensitive(AnimationOptions)
    def animation(self, value):
        self._animation = value

    @property
    def border_color(self) -> Optional[str | Gradient | Pattern]:
        """A specific border color for the selected point. Defaults to ``'#000000'``.

        :returns: The border color for the hovered point.
        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._border_color

    @border_color.setter
    def border_color(self, value):
        from highcharts_core import utility_functions
        self._border_color = utility_functions.validate_color(value)

    @property
    def color(self) -> Optional[str | Gradient | Pattern]:
        """A specific color for the selected point. Defaults to ``'#cccccc'``.

        :rtype: :class:`str <python:str>`, :class:`Gradient`, :class:`Pattern`, or
          :obj:`None <python:None>`
        """
        return self._color

    @color.setter
    def color(self, value):
        from highcharts_core import utility_functions
        self._color = utility_functions.validate_color(value)

    @property
    def enabled(self) -> Optional[bool]:
        """Enable separate styles for the hovered series to visualize that the user hovers
        either the series itself or the legend. Defaults to ``True``.

        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._enabled

    @enabled.setter
    def enabled(self, value):
        if value is None:
            self._enabled = None
        else:
            self._enabled = bool(value)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'animation': as_dict.get('animation', None),
            'border_color': as_dict.get('borderColor', None),
            'color': as_dict.get('color', None),
            'enabled': as_dict.get('enabled', None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'animation': self.animation,
            'borderColor': self.border_color,
            'color': self.color,
            'enabled': self.enabled
        }

        return untrimmed


class States(HighchartsMeta):
    """Collection of state configuration settings that can be applied to series or
    markers."""

    def __init__(self, **kwargs):
        self._hover = None
        self._inactive = None
        self._normal = None
        self._select = None

        self.hover = kwargs.get('hover', None)
        self.inactive = kwargs.get('inactive', None)
        self.normal = kwargs.get('normal', None)
        self.select = kwargs.get('select', None)

    @property
    def hover(self) -> Optional[HoverState]:
        """Options for the hovered point/series.

        .. note::

          These settings override the normal state  options when a point/series is moused
          over or touched.

        :rtype: :class:`HoverState` or :obj:`None <python:None>`
        """
        return self._hover

    @hover.setter
    @class_sensitive(HoverState)
    def hover(self, value):
        self._hover = value

    @property
    def inactive(self) -> Optional[InactiveState]:
        """The opposite state of a hover for a series/point.

        :rtype: :class:`InactiveState` or :obj:`None <python:None>`
        """
        return self._inactive

    @inactive.setter
    @class_sensitive(InactiveState)
    def inactive(self, value):
        self._inactive = value

    @property
    def normal(self) -> Optional[NormalState]:
        """The normal state of a series, or for point items in column, pie and similar
        series.

        Currently only used for setting animation when returning to normal state from
        hover.

        :rtype: :class:`NormalState` or :obj:`None <python:None>`
        """
        return self._normal

    @normal.setter
    @class_sensitive(NormalState)
    def normal(self, value):
        self._normal = value

    @property
    def select(self) -> Optional[SelectState]:
        """Options for the selected point.

        These settings override the normal state options when a point is selected.

        :rtype: :class:`SelectState` or :obj:`None <python:None>`
        """
        return self._select

    @select.setter
    @class_sensitive(SelectState)
    def select(self, value):
        self._select = value

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'hover': as_dict.get('hover', None),
            'inactive': as_dict.get('inactive', None),
            'normal': as_dict.get('normal', None),
            'select': as_dict.get('select', None)
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'hover': self.hover,
            'inactive': self.inactive,
            'normal': self.normal,
            'select': self.select
        }

        return untrimmed



from typing import Optional, List

from validator_collection import validators, checkers

from highcharts_core.metaclasses import HighchartsMeta
from highcharts_core.decorators import class_sensitive, validate_types
from highcharts_core.options.sonification.track_configurations import (InstrumentTrackConfiguration,
                                                                       SpeechTrackConfiguration,
                                                                       ContextTrackConfiguration)
from highcharts_core.options.sonification.grouping import SonificationGrouping


class SeriesSonification(HighchartsMeta):
    """Sonification/audio chart options for a series."""
    
    def __init__(self, **kwargs):
        self._context_tracks = None
        self._default_instrument_options = None
        self._default_speech_options = None
        self._enabled = None
        self._point_grouping = None
        self._tracks = None
        
        self.context_tracks = kwargs.get('context_tracks', None)
        self.default_instrument_options = kwargs.get('default_instrument_options', None)
        self.default_speech_options = kwargs.get('default_speech_options', None)
        self.enabled = kwargs.get('enabled', None)
        self.point_grouping = kwargs.get('point_grouping', None)
        self.tracks = kwargs.get('tracks', None)
        
    @property
    def _dot_path(self) -> Optional[str]:
        """The dot-notation path to the options key for the current class.
        
        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return 'plotOptions.series.sonification'

    @property
    def context_tracks(self) -> Optional[ContextTrackConfiguration | List[ContextTrackConfiguration]]:
        """Context tracks for the series. Context tracks are not tied to data points.
        
        :rtype: :class:`ContextTrackConfiguration <highcharts_core.options.sonification.track_configurations.ContextTrackConfiguration>` 
          or :class:`list <python:list>` of track configuration types
        """
        return self._context_tracks
    
    @context_tracks.setter
    def context_tracks(self, value):
        if not value:
            self._context_tracks = None
        elif checkers.is_iterable(value, forbid_literals = (str, bytes, dict)):
                self._context_tracks = [validate_types(x, types = (ContextTrackConfiguration)) for x in value]
        else:
            value = validate_types(value, types = ContextTrackConfiguration)
            self._context_tracks = value
    
    @property
    def default_instrument_options(self) -> Optional[InstrumentTrackConfiguration]:
        """Default sonification options for all instrument tracks.

        .. warning::
        
          If specific options are also set on individual tracks or per-series, this configuration will be *overridden*.
        
        :rtype: :class:`InstrumentTrackConfiguration <highcharts_core.options.sonification.track_configurations.InstrumentTrackConfiguration>`
          or :obj:`None <python:None>`
        """
        return self._default_instrument_options
    
    @default_instrument_options.setter
    @class_sensitive(InstrumentTrackConfiguration)
    def default_instrument_options(self, value):
        self._default_instrument_options = value
        
    @property
    def default_speech_options(self) -> Optional[SpeechTrackConfiguration]:
        """Default sonification options for all speech tracks.
        .. warning::
        
          If specific options are also set on individual tracks or per-series, this configuration will be *overridden*.
        
        :rtype: :class:`SpeechTrackConfiguration <highcharts_core.options.sonification.track_configurations.SpeechTrackConfiguration>`
          or :obj:`None <python:None>`
        """
        return self._default_speech_options
    
    @default_speech_options.setter
    @class_sensitive(SpeechTrackConfiguration)
    def default_speech_options(self, value):
        self._default_speech_options = value
        
    @property
    def enabled(self) -> Optional[bool]:
        """If ``True``, sonification will be enabled for the series.
        
        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._enabled
    
    @enabled.setter
    def enabled(self, value):
        if value is None:
            self._enabled = None
        else:
            self._enabled = bool(value)
            
    @property
    def point_grouping(self) -> Optional[SonificationGrouping]:
        """Options for grouping data points together when sonifying. 
        
        This allows for the visual presentation to contain more points than what is being played. 
        
        If not enabled, all visible / uncropped points are played.
        
        :rtype: :class:`SonificationGrouping <highcharts_core.options.sonification.grouping.SonificationGrouping>` or
          :obj:`None <python:None>`
        """
        return self._point_grouping
    
    @point_grouping.setter
    @class_sensitive(SonificationGrouping)
    def point_grouping(self, value):
        self._point_grouping = value
        
    @property
    def tracks(self) -> Optional[ContextTrackConfiguration | List[ContextTrackConfiguration]]:
        """Tracks for the series.
        
        :rtype: :class:`ContextTrackConfiguration <highcharts_core.options.sonification.track_configurations.ContextTrackConfiguration>` 
          or :class:`list <python:list>` of 
          :class:`ContextTrackConfiguration <highcharts_core.options.sonification.track_configurations.ContextTrackConfiguration>`
          or :obj:`None <python:None>`
        """
        return self._tracks
    
    @tracks.setter
    def tracks(self, value):
        if not value:
            self._tracks = None
        elif checkers.is_iterable(value, forbid_literals = (str, bytes, dict)):
            self._tracks = [validate_types(x, types = (ContextTrackConfiguration)) for x in value]
        else:
            self._tracks = validate_types(value, types = (ContextTrackConfiguration))

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'context_tracks': as_dict.get('contextTracks', None),
            'default_instrument_options': as_dict.get('defaultInstrumentOptions', None),
            'default_speech_options': as_dict.get('defaultSpeechOptions', None),
            'enabled': as_dict.get('enabled', None),
            'point_grouping': as_dict.get('pointGrouping', None),
            'tracks': as_dict.get('tracks', None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'contextTracks': self.context_tracks,
            'defaultInstrumentOptions': self.default_instrument_options,
            'defaultSpeechOptions': self.default_speech_options,
            'enabled': self.enabled,
            'pointGrouping': self.point_grouping,
            'tracks': self.tracks,
        }

        return untrimmed



"""Tests for ``highcharts.responsive``."""

import pytest

from json.decoder import JSONDecodeError

from highcharts_core.options.responsive import Responsive as cls
from highcharts_core import errors
from tests.fixtures import input_files, check_input_file, to_camelCase, to_js_dict, \
    Class__init__, Class__to_untrimmed_dict, Class_from_dict, Class_to_dict, \
    Class_from_js_literal

STANDARD_PARAMS = [
    ({}, None),
    ({
      'rules': [
          {
           'chart_options': None,
           'condition': {
               'maxHeight': 20,
               'minHeight': 10,
               'maxWidth': 30,
               'minWidth': 10
           }
          }
      ]
     }, None),
]


@pytest.mark.parametrize('kwargs, error', STANDARD_PARAMS)
def test__init__(kwargs, error):
    Class__init__(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error', STANDARD_PARAMS)
def test__to_untrimmed_dict(kwargs, error):
    Class__to_untrimmed_dict(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error',  STANDARD_PARAMS)
def test_from_dict(kwargs, error):
    Class_from_dict(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error',  STANDARD_PARAMS)
def test_to_dict(kwargs, error):
    Class_to_dict(cls, kwargs, error)


@pytest.mark.parametrize('filename, as_file, error', [
    ('responsive/01.js', False, None),
    ('responsive/02.js', False, None),

    ('responsive/error-01.js', False, (errors.HighchartsValueError,
                                       errors.HighchartsParseError,
                                       JSONDecodeError)),
    ('responsive/error-02.js', False, (errors.HighchartsValueError,
                                       errors.HighchartsParseError,
                                       JSONDecodeError)),

    ('responsive/01.js', True, None),
    ('responsive/02.js', True, None),

    ('responsive/error-01.js', True, (errors.HighchartsValueError,
                                      errors.HighchartsParseError,
                                      JSONDecodeError)),
    ('responsive/error-02.js', True, (errors.HighchartsValueError,
                                      errors.HighchartsParseError,
                                      JSONDecodeError)),
])
def test_from_js_literal(input_files, filename, as_file, error):
    Class_from_js_literal(cls, input_files, filename, as_file, error)



from typing import Optional
from decimal import Decimal

from validator_collection import validators

from highcharts_core.metaclasses import HighchartsMeta


class Jitter(HighchartsMeta):
    """Apply a jitter effect for the rendered markers.

    When plotting discrete values, a little random noise may help telling the points
    apart. The jitter setting applies a random displacement of up to n axis units in
    either direction.

    So for example on a horizontal X axis, setting the ``jitter.x`` to ``0.24`` will
    render the point in a random position between 0.24 units to the left and 0.24
    units to the right of the true axis position. On a category axis, setting it to
    ``0.5`` will fill up the bin and make the data appear continuous.

    When rendered on top of a box plot or a column series, a jitter value of 0.24 will
    correspond to the underlying series' default ``group_padding`` and
    ``point_padding`` settings.

    """

    def __init__(self, **kwargs):
        self._x = None
        self._y = None

        self.x = kwargs.get('x', None)
        self.y = kwargs.get('y', None)

    @property
    def x(self) -> Optional[int | float | Decimal]:
        """The maximal X offset for the random jitter effect. Defaults to ``0``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._x

    @x.setter
    def x(self, value):
        self._x = validators.numeric(value, allow_empty = True)

    @property
    def y(self) -> Optional[int | float | Decimal]:
        """The maximal Y offset for the random jitter effect. Defaults to ``0``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._y

    @y.setter
    def y(self, value):
        self._y = validators.numeric(value, allow_empty = True)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        return {
            'x': as_dict.get('x', None),
            'y': as_dict.get('y', None)
        }

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        return {
            'x': self.x,
            'y': self.y
        }
